/***************************************************************************
                          arraydlg.cpp  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sun Apr 25 1999
    copyright            : (C) 1999 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <qfileinfo.h>
#include <qcheckbox.h>
#include <qspinbox.h>
#include <qlineedit.h>
#include <kapp.h>
#include <kurl.h>
#include <kmsgbox.h>
#include <kfiledialog.h>
#include <kseparator.h>
#include <kcolorbtn.h>
#include <krestrictedline.h>
#include "arraydlg.h"
#include "kpldoc.h"
#include "symboldlg.h"
#include "utils.h"

#ifndef min
  #define min(x,y) ((x) < (y) ? (x) : (y))
#endif
#ifndef max
  #define max(x,y) ((x) > (y) ? (x) : (y))
#endif

ArrayDlg::ArrayDlg(QWidget *parent, KplDoc *model, PARRAYDATA ad0) :
 QDialog(parent, 0, true)
{
  QString s;

  m = model;
  ad = ad0;
  adt = 0;
  setCaption(i18n("Array"));
  fileName = new QLabel(ad->path, this);
  fileName->setGeometry(110, 12, 290, 22);
  fileName->setFrameStyle(QFrame::Panel | QFrame::Sunken);
  QPushButton *b = new QPushButton(i18n("File"), this);
  b->setGeometry(10, 11, 90, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotFile()));
  QLabel *l = new QLabel(i18n("x column"), this);
  l->setGeometry(10, 46, 86, 22);
  ix = new QSpinBox(0, 0, 1, this);
  ix->setGeometry(110, 46, 50, 22);
  l = new QLabel(i18n("y column"), this);
  l->setGeometry(250, 46, 100, 22);
  iy = new QSpinBox(0, 0, 1, this);
  iy->setGeometry(350, 46, 50, 22);
  l = new QLabel(i18n("Error column"), this);
  l->setGeometry(10, 80, 86, 22);
  ie = new QSpinBox(0, 0, 1, this);
  ie->setGeometry(110, 80, 50, 22);
  err = new QCheckBox(i18n("Error bar"), this);
  err->setGeometry(175, 80, 120, 22);
  err->setChecked(ad->errbars);
  l = new QLabel(i18n("Start index"), this);
  l->setGeometry(10, 114, 86, 22);
  iStart = new QSpinBox(0, 0, 1, this);
  iStart->setGeometry(110, 114, 50, 22);
  l = new QLabel(i18n("Number of points"), this);
  l->setGeometry(250, 114, 100, 22);
  n = new QSpinBox(0, 0, 1, this);
  n->setGeometry(350, 114, 50, 22);
  l = new QLabel(i18n("x normalization"), this);
  l->setGeometry(10, 148, 88, 22);
  efx = new QLineEdit(this);
  efx->setGeometry(110, 148, 50, 22);
  efx->setText(s.sprintf("%7g", ad->fx));
  l = new QLabel(i18n("y normalization"), this);
  l->setGeometry(250, 148, 100, 22);
  efy = new QLineEdit(this);
  efy->setGeometry(350, 148, 50, 22);
  efy->setText(s.sprintf("%7g", ad->fy));
  l = new QLabel(i18n("Symbol"), this);
  l->setGeometry(10, 182, 86, 22);
  symb = new QSpinBox(-17, 9, 1, this);
  symb->setGeometry(110, 182, 50, 22);
  symb->setValue(ad->symb);
  b = new QPushButton("...", this);
  b->setGeometry(175, 181, 25, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotSymbol()));
  l = new QLabel(i18n("Color"), this);
  l->setGeometry(250, 182, 100, 22);
  colData = new KColorButton(this, 0);
  colData->setGeometry(350, 181, 50, 25);
  colData->setColor(ad->color);
  KSeparator *hl = new KSeparator(KSeparator::HLine, this);
  hl->setGeometry(10, 216, 390, 0);
  b = new QPushButton(i18n("OK"), this);
  b->setGeometry(10, 228, 90, 25);
  b->setDefault(true);
  connect(b, SIGNAL(clicked()), SLOT(slotOK()));
  b = new QPushButton(i18n("Apply"), this);
  b->setGeometry(110, 228, 90, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotApply()));
  b = new QPushButton(i18n("Cancel"), this);
  b->setGeometry(210, 228, 90, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotCancel()));
  b = new QPushButton(i18n("Help"), this);
  b->setGeometry(310, 228, 90, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotHelp()));
  adjustSize();
  setFixedSize(size());
  setEditArrayControls(ad);
}

void ArrayDlg::getValues(bool ok)
{
  if (!getVal(this, efx, &ad->fx, i18n("invalid fx!")))
    return;
  if (!getVal(this, efy, &ad->fy, i18n("invalid fy!")))
    return;
  ad->color = colData->color().rgb();
  ad->ix = ix->value();
  ad->iy = iy->value();
  ad->ie = ie->value();
  ad->symb = symb->value();
  ad->istart = iStart->value();
  ad->n = n->value();
  strcpy(ad->path, fileName->text());
  ad->errbars = err->isChecked();
  if (adt) {
    m->freeXCond(ad);
    ad->x = adt->x;
    ad->ncols = adt->ncols;
    ad->nrows = adt->nrows;
    free(adt);
    adt = 0;
  }
  ad->istart = max(ad->istart, 0);
  ad->n = min(ad->n, ad->nrows - ad->istart);
  m->setModified();
  emit modelChanged();
  if (ok)
    accept();
  else
    setEditArrayControls(ad);
}

void ArrayDlg::setEditArrayControls(PARRAYDATA ad)
{
  int iColMax = max(0, ad->ncols - 1);
  ix->setRange(0, iColMax);
  ix->setValue(ad->ix);
  iy->setRange(0, iColMax);
  iy->setValue(ad->iy);
  ie->setRange(0, iColMax);
  ie->setValue(ad->ie);
  iStart->setRange(0, max(0, ad->nrows - 1));
  iStart->setValue(ad->istart);
  n->setRange(0, ad->nrows);
  n->setValue(ad->n);
}

void ArrayDlg::slotSymbol()
{
  SymbolDlg dlg(this, symb);
  dlg.exec();
}

void ArrayDlg::slotFile()
{
  QString url = KFileDialog::getOpenFileURL(m->currentDir(), "*.dat");
  if (!url.isEmpty()) {
    KURL u = KURL(url);
    if (u.isLocalFile())
      url = u.path();
    if (strcmp(url, fileName->text())) {
      adt = (PARRAYDATA) calloc(1, sizeof(ARRAYDATA));
      if (m->readFile(url, &adt->x, &adt->ncols, &adt->nrows)) {
        adt->ix = min(ad->ix, adt->ncols - 1);
        adt->iy = min(ad->iy, adt->ncols - 1);
        adt->ie = min(ad->ie, adt->ncols - 1);
        adt->istart = min(ad->istart, adt->nrows - 1);
        adt->n = adt->nrows - adt->istart;
        if (ad->n)
          adt->n = min(ad->n, adt->n);
        setEditArrayControls(adt);
        fileName->setText(url);
        if (u.isLocalFile()) {
          QFileInfo fi(url);
          m->setCurrentDir(fi.dirPath(true));
        } else
          m->setCurrentDir(u.directoryURL());
      } else {
        free(adt);
        adt = 0;
        KMsgBox::message(this, i18n("Error"), i18n("while loading the file"),
                         KMsgBox::EXCLAMATION);
      }
    }
  }
}

void ArrayDlg::slotOK()
{
  getValues(true);
}

void ArrayDlg::slotApply()
{
  getValues(false);
}

void ArrayDlg::slotCancel()
{
  if (adt) {
    m->freeX(&adt->x);
    free(adt);
  }
  reject();
}

void ArrayDlg::slotHelp()
{
  kapp->invokeHTMLHelp("kpl/index-5.html", "sec-array");
}

