/***************************************************************************
                          autofitdlg.h  -  description                              
                             -------------------                                         
    begin                : Sat Jun 5 1999                                           
    copyright            : (C) 1999 by Werner Stille                         
    email                : stille@uni-freiburg.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#ifndef AUTOFITDLG_H
#define AUTOFITDLG_H

#include <qdialog.h>
#include "kplots.h"

class QLineEdit;
class QSpinBox;
class QCheckBox;
class KColorButton;
class KplDoc;

/**
  * Edit fit parameter dialog class. Provides a dialog for the editing
  * the parameters of a function.
  * @author Werner Stille
  */
class EditFitParameterDlg : public QDialog {
  Q_OBJECT

public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param p parameter array.
   * @param bFit array indicating whether a parameter is to be fitted or fixed.
   * @param file absolute path of the function library.
   * @param func function name.
   */
  EditFitParameterDlg(QWidget *parent, double *p, bool *bFit, const char *file,
                      const char *func);

protected slots:
  /** Accepts changes and leaves the dialog. */
  void slotOK();

private:
  bool *bFit;
  double *p;
  QLineEdit *par[NPMAX];
  QCheckBox *enFit[NPMAX];
};

/**
  * Autofit settings dialog class. Provides a dialog for the
  * settings of the automatic parameter fit of a function to a data set from a file.
  * @author Werner Stille
  */
class AutoFitDlg : public QDialog {
  Q_OBJECT

public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param model pointer to KplDoc model object.
   */
	AutoFitDlg(QWidget *parent, KplDoc *model);
  /** Destructor. */
	~AutoFitDlg();

protected slots:
  /** Displays the dialog for selection of a library file to load. */
  void slotFile();
  /** Displays the dialog for selection of a function. */
  void slotFunc();
  /** Displays the dialog for selection of a parameter file to read. */
  void slotGetParameter();
  /** Displays the dialog for parameter editing. */
  void slotEditParameter();
  /** Displays the dialog for selection of a parameter file to save. */
  void slotSaveParameter();
  /** Displays the dialog for setting the calculation of standard deviations. */
  void slotErrMod();
  /** Displays the dialog for selection of lines and markers. */
  void slotSymbol();
  /** Applies settings and leaves the dialog. */
  void slotOK();
  /** Displays help for the function settings dialog. */
  void slotHelp();

private:
  KplDoc *m;
  AutoStruct *aut;
  QLineEdit *fileName, *func, *eTol;
  QSpinBox *symb, *sItmax;
  KColorButton *colData;
  QCheckBox *errCol, *showDlg, *saveRes, *follow;
};

#endif
