/***************************************************************************
                          basic.cpp  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sun Apr 25 1999
    copyright            : (C) 1999 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qgroupbox.h>
#include <qlineedit.h>
#include <kapp.h>
#include <kmsgbox.h>
#include <kcolorbtn.h>
#include "basicdlg.h"
#include "utils.h"

BasicDlg::BasicDlg(QWidget *parent, BasicStruct *bs) : QDialog(parent, 0, true)
{
  QString s;

  b = bs;
  setCaption(i18n("Basic settings"));
  QGroupBox *g = new QGroupBox(i18n("Format "), this);
  g->setGeometry(12, 6, 274, 112);
  QLabel *l = new QLabel(i18n("Width"), this);
  l->setGeometry(22, 36, 46, 22);
  formatx = new QLineEdit(this);
  formatx->setGeometry(68, 36, 50, 22);
  formatx->setText(s.setNum(b->xf, 'f', 2));
  l = new QLabel("cm", this);
  l->setGeometry(122, 36, 30, 22);
  l = new QLabel(i18n("Height"), this);
  l->setGeometry(156, 36, 44, 22);
  formaty = new QLineEdit(this);
  formaty->setGeometry(200, 36, 50, 22);
  formaty->setText(s.setNum(b->yf, 'f', 2));
  l = new QLabel("cm", this);
  l->setGeometry(254, 36, 30, 22);
  QPushButton *bn = new QPushButton(i18n("Portrait A4"), this);
  bn->setGeometry(28, 76, 110, 25);
  connect(bn, SIGNAL(clicked()), SLOT(slotPortrait()));
  bn = new QPushButton(i18n("Landscape A4"), this);
  bn->setGeometry(158, 76, 110, 25);
  connect(bn, SIGNAL(clicked()), SLOT(slotLandscape()));
  g = new QGroupBox(i18n("Default colors"), this);
  g->setGeometry(12, 124, 274, 130);
  l = new QLabel(i18n("Frame"), this);
  l->setGeometry(22, 149, 86, 25);
  colFrame = new KColorButton(this, 0);
  colFrame->setGeometry(108, 149, 80, 25);
  colFrame->setColor(QColor(b->colFrame));
  l = new QLabel(i18n("Grid"), this);
  l->setGeometry(22, 181, 86, 25);
  colGrid = new KColorButton(this, 0);
  colGrid->setGeometry(108, 181, 80, 25);
  colGrid->setColor(QColor(bs->colGrid));
  l = new QLabel(i18n("Data"), this);
  l->setGeometry(22, 213, 86, 25);
  colData = new KColorButton(this, 0);
  colData->setGeometry(108, 213, 80, 25);
  colData->setColor(QColor(bs->colData));
  bn = new QPushButton(i18n("OK"), this);
  bn->setGeometry(12, 266, 80, 25);
  bn->setDefault(true);
  connect(bn, SIGNAL(clicked()), SLOT(slotOK()));
  bn = new QPushButton(i18n("Cancel"), this);
  bn->setGeometry(102, 266, 80, 25);
  connect(bn, SIGNAL(clicked()), SLOT(reject()));
  bn = new QPushButton(i18n("Help"), this);
  bn->setGeometry(204, 266, 80, 25);
  connect(bn, SIGNAL(clicked()), SLOT(slotHelp()));
  adjustSize();
  setFixedSize(size());
}

void BasicDlg::slotPortrait()
{
  formatx->setText("21.00");
  formaty->setText("29.70");
}

void BasicDlg::slotLandscape()
{
  formatx->setText("29.70");
  formaty->setText("21.00");
}

void BasicDlg::slotOK()
{
  double xf, yf;
  if (!getVal(this, formatx, &xf, i18n("invalid width!")))
    return;
  if (!getVal(this, formaty, &yf, i18n("invalid height!")))
    return;
  b->xf = xf;
  b->yf = yf;
  b->colFrame = colFrame->color().rgb();
  b->colGrid = colGrid->color().rgb();
  b->colData = colData->color().rgb();
  accept();
}

void BasicDlg::slotHelp()
{
  kapp->invokeHTMLHelp("kpl/index-5.html", "sec-basic");
}
