/***************************************************************************
                          edittextdlg.cpp  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sun Apr 25 1999
    copyright            : (C) 1999 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qpushbutton.h>
#include <qlineedit.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <kapp.h>
#include "edittextdlg.h"

Preview::Preview(QWidget *parent) : QWidget(parent)
{
  QWidget::setBackgroundColor(white);
  str[0] = 0;
}

void Preview::setText(const char *s)
{
  strcpy(str, s);
  update();
}

void Preview::paintEvent(QPaintEvent *)
{
  plInit(this, 0, 0);
  relsize(2.0);
  text(str, 10, 20, 1);
  end();
}

EditTextDlg::EditTextDlg(QWidget *parent, QLineEdit *e) :
 QDialog(parent, 0, true)
{
  ed = e;
  setCaption(i18n("Edit text"));
  strcpy(sfont, "#NN");
  editPreview = new Preview(this);
  editPreview->setGeometry(8, 12, 276, 40);
  editPreview->setText(ed->text());
  text = new QLineEdit(this);
  text->setGeometry(8, 62, 174, 24);
  text->setText(ed->text());
  text->setCursorPosition(strlen(ed->text()));
  connect(text, SIGNAL(textChanged(const char *)),
          editPreview, SLOT(setText(const char *)));
  font = new QLineEdit(this);
  font->setGeometry(228, 62, 56, 24);
  font->setText(sfont);
  QPushButton *b = new QPushButton("<", this);
  b->setGeometry(194, 62, 25, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotCat()));
  bg1 = new QButtonGroup(i18n("Font"), this);
  bg1->setGeometry(8, 92, 174, 176);
  QRadioButton *r = new QRadioButton("Helvetica", bg1);
  r->setGeometry(18, 20, 150, 24);
  connect(r, SIGNAL(clicked()), SLOT(slotFont()));
  r->setChecked(true);
  r = new QRadioButton("Helvetica Bold", bg1);
  r->setGeometry(18, 44, 150, 24);
  connect(r, SIGNAL(clicked()), SLOT(slotFont()));
  r = new QRadioButton("Helvetica Italic", bg1);
  r->setGeometry(18, 68, 150, 24);
  connect(r, SIGNAL(clicked()), SLOT(slotFont()));
  r = new QRadioButton("Helvetica Bold Italic", bg1);
  r->setGeometry(18, 92, 150, 24);
  connect(r, SIGNAL(clicked()), SLOT(slotFont()));
  r = new QRadioButton("Symbol", bg1);
  r->setGeometry(18, 116, 150, 24);
  connect(r, SIGNAL(clicked()), SLOT(slotFont()));
  r = new QRadioButton("ZapfDingbats", bg1);
  r->setGeometry(18, 140, 150, 24);
  connect(r, SIGNAL(clicked()), SLOT(slotFont()));
  bg2 = new QButtonGroup(i18n("Position"), this);
  bg2->setGeometry(194, 92, 90, 98);
  r = new QRadioButton(i18n("Up"), bg2);
  r->setGeometry(12, 20, 75, 24);
  connect(r, SIGNAL(clicked()), SLOT(slotPos()));
  r = new QRadioButton(i18n("Normal"), bg2);
  r->setGeometry(12, 44, 75, 24);
  connect(r, SIGNAL(clicked()), SLOT(slotPos()));
  r->setChecked(true);
  r = new QRadioButton(i18n("Down"), bg2);
  r->setGeometry(12, 68, 75, 24);
  connect(r, SIGNAL(clicked()), SLOT(slotPos()));
  b = new QPushButton(i18n("OK"), this);
  b->setGeometry(194, 202, 90, 25);
  b->setDefault(true);
  connect(b, SIGNAL(clicked()), SLOT(slotOK()));
  b = new QPushButton(i18n("Cancel"), this);
  b->setGeometry(194, 240, 90, 25);
  connect(b, SIGNAL(clicked()), SLOT(reject()));
  adjustSize();
  setFixedSize(size());
}

void EditTextDlg::slotCat()
{
  text->insert(font->text());
}

void EditTextDlg::slotFont()
{
  int i = -1;
  char fs[] = "HBOISZ";
  QRadioButton *rb;

  do 
    rb = (QRadioButton *) bg1->find(++i);
  while (!rb->isChecked());
  sfont[1] = fs[i];
  font->setText(sfont);
}

void EditTextDlg::slotPos()
{
  int i = -1;
  char ps[] = "UND";
  QRadioButton *rb;

  do 
    rb = (QRadioButton *) bg2->find(++i);
  while (!rb->isChecked());
  sfont[2] = ps[i];
  font->setText(sfont);
}

void EditTextDlg::slotOK()
{
  ed->setText(text->text());
  accept();
}
