/***************************************************************************
                          fitdlg.cpp  -  description                              
                             -------------------                                         
    begin                : Tue May 25 1999                                           
    copyright            : (C) 1999 by Werner Stille                         
    email                : stille@uni-freiburg.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include <stdlib.h>
#include <dlfcn.h>
#include <qfileinfo.h>
#include <qlistbox.h>
#include <qspinbox.h>
#include <qlineedit.h>
#include <qmultilinedit.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qpainter.h>
#include <qdir.h>
#include <kapp.h>
#include <kfm.h>
#include <kmsgbox.h>
#include <kfiledialog.h>
#include <kcolorbtn.h>
#include <kseparator.h>
#include "funcdlg.h"
#include "fitdlg.h"
#include "lmfit.h"
#include "kpldoc.h"
#include "utils.h"

#ifndef min
  #define min(x,y) ((x) < (y) ? (x) : (y))
#endif
#ifndef max
  #define max(x,y) ((x) > (y) ? (x) : (y))
#endif

#define CELL_WIDTH 50
#define CELL_HEIGHT 30

CorrTable::CorrTable(QWidget *parent, int np0, int *ip0, double *corr0) :
  QTableView(parent)
{
  np = np0;
  ip = ip0;
  corr = corr0;
  int n = np + 1;
  setBackgroundColor(kapp->windowColor);
  setNumCols(n);
  setNumRows(n);
  setCellWidth(CELL_WIDTH);
  setCellHeight(CELL_HEIGHT);
  setTableFlags(Tbl_autoScrollBars | Tbl_clipCellPainting | Tbl_smoothScrolling);
  resize(min(n * cellWidth(), 600), min(n * cellHeight(), 400));
  connect(kapp, SIGNAL(kdisplayPaletteChanged()), SLOT(slotSetBackground()));
}

void CorrTable::paintCell(QPainter *p, int row, int col)
{
  double temp;
  QString s;

  if (row)
    if (col) {
      temp=corr[np * (row - 1) + col - 1];
      if ((temp >= -1.0004) && (temp  <= 1.0004))
        s.setNum(temp, 'f', 3);
      else
        s = "NaN";
    } else
      s.sprintf("p%i", ip[row - 1]);
  else
    if (col)
      s.sprintf("p%i", ip[col - 1]);
  p->setPen(kapp->windowTextColor);
  p->drawText(0, 0, cellWidth(), cellHeight(), AlignCenter, s);
}

void CorrTable::slotSetBackground(void)
{
  setBackgroundColor(kapp->windowColor);
}

CorrDlg::CorrDlg(QWidget *parent, int np, int *ip, double *corr) :
  QDialog(parent, 0, true)
{
  setCaption(i18n("Parameter correlation coefficients"));
  table = new CorrTable(this, np, ip, corr);
  adjustSize();
  int n = np + 1;
  setMinimumSize(2 * CELL_WIDTH, 2 * CELL_HEIGHT);
  setMaximumSize(n * CELL_WIDTH, n * CELL_HEIGHT);
}

void CorrDlg::resizeEvent(QResizeEvent *ev)
{
  table->resize(ev->size());
}

ErrModDlg::ErrModDlg(QWidget *parent, KplDoc *model, AutoStruct *a0) :
  QDialog(parent, 0, true)
{
  QString s;

  m = model;
  a = a0;
  at = new AutoStruct;
  *at = *a;
  setCaption(i18n("Error model Function"));
  QLabel *l = new QLabel(i18n("Library"), this);
  l->setGeometry(10, 10, 90, 22);
  fileName = new QLineEdit(this);
  fileName->setGeometry(105, 10, 240, 22);
  fileName->setText(a->errModPath);
  QPushButton *b = new QPushButton("...", this);
  b->setGeometry(355, 9, 25, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotFile()));
  l = new QLabel(i18n("Function"), this);
  l->setGeometry(10, 50, 90, 22);
  func = new QLineEdit(this);
  func->setGeometry(105, 50, 240, 22);
  func->setText(a->errModName);
  b = new QPushButton("...", this);
  b->setGeometry(355, 49, 25, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotFunc()));
  l = new QLabel(i18n("Parameter"), this);
  l->setGeometry(10, 90, 90, 22);
  b = new QPushButton(i18n("Edit"), this);
  b->setGeometry(105, 89, 85, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotEditParameter()));
  b = new QPushButton(i18n("Load"), this);
  b->setGeometry(200, 89, 85, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotGetParameter()));
  b = new QPushButton(i18n("Save"), this);
  b->setGeometry(295, 89, 85, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotSaveParameter()));
  KSeparator *hl = new KSeparator(KSeparator::HLine, this);
  hl->setGeometry(10, 120, 370, 0);
  b = new QPushButton(i18n("OK"), this);
  b->setGeometry(10, 130, 85, 25);
  b->setDefault(true);
  connect(b, SIGNAL(clicked()), SLOT(slotOK()));
  b = new QPushButton(i18n("Cancel"), this);
  b->setGeometry(105, 130, 85, 25);
  connect(b, SIGNAL(clicked()), SLOT(reject()));
  b = new QPushButton(i18n("Help"), this);
  b->setGeometry(295, 130, 85, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotHelp()));
  adjustSize();
  setFixedSize(size());
}

ErrModDlg::~ErrModDlg()
{
  delete at;
}

void ErrModDlg::slotFile()
{
  getFile(fileName, m);
}

void ErrModDlg::slotFunc()
{
  FILE *f;

  QFileInfo fi(fileName->text());
  QString path = fi.dirPath(true);
  if ((f = fopen(path + "/" + fi.baseName() + ".def", "r"))) {
    ChooseFuncDlg dlg(this, f, func);
    dlg.exec();
  } else
    KMsgBox::message(this, i18n("Error"),
                     i18n("while trying to open module definition file"),
                     KMsgBox::EXCLAMATION);
}

void ErrModDlg::slotGetParameter()
{
  getPar(this, at->pErrMod, m);
}

void ErrModDlg::slotEditParameter()
{
  if (m->aut.showSource) {
    EditParameterDlg dlg(this, at->pErrMod, fileName->text(), func->text());
    dlg.exec();
  } else {
    EditParameterDlg dlg(this, at->pErrMod, 0, 0);
    dlg.exec();
  }
}

void ErrModDlg::slotSaveParameter()
{
  savePar(this, at->pErrMod, m);
}

void ErrModDlg::slotOK()
{
  char *error;
  void *hErrMod = 0;

  strcpy(at->errModPath, fileName->text());
  strcpy(at->errModName, func->text());
  if (strcmp(a->errModPath, at->errModPath) ||
      strcmp(a->errModName, at->errModName)) {
    if (at->errModPath[0]) {
      if (!(hErrMod = dlopen(at->errModPath, RTLD_NOW))) {
        QString msg;
        msg.sprintf(i18n("while trying to open\nfile %s:\n%s"),
                    at->errModPath, dlerror());
        KMsgBox::message(this, i18n("Error"), msg,
                         KMsgBox::EXCLAMATION);
        return;
      }
    } else
      if (at->errModName[0]) {
        KMsgBox::message(this, i18n("Error"), i18n("no library specified!"));
        return;
      }
    if (at->errModName[0]) {
      dlsym(hErrMod, at->errModName);
      if ((error = (char *) dlerror()) != 0) {
        QString msg;
        msg.sprintf(i18n("while trying to determine the address of\nfunction %s:\n%s"),
                    at->errModName, error);
        KMsgBox::message(this, i18n("Error"), msg,
                         KMsgBox::EXCLAMATION);
        dlclose(hErrMod);
        return;
      }
    }
  }
  if (hErrMod)
    dlclose(hErrMod);
  *a = *at;
  accept();
}

void ErrModDlg::slotHelp()
{
  kapp->invokeHTMLHelp("kpl/index-5.html", "sec-errormodel");
}

FitDlg::FitDlg(QWidget *parent, KplDoc *model, PARRAYDATA ad0,
               PFUNDATA fd0, int mode) : QDialog(parent, 0, true)
{
  QString s;
  QLabel *l;
  int i, x, y;

  m = model;
  ad = ad0;
  fd = fd0;
  dlgMode = mode;
  aut = new AutoStruct;
  *aut = m->aut;
  for (i = 0; i < NPMAX; i++) {
    p[i] = fd->p[i];
    pErr[i] = 0.0;
  }
  if (dlgMode & SHOWDLG) {
    setCaption(i18n("Parameter fit"));
    for (i = 0; i < NPMAX; i++) {
      x = 262 * (i / 10);
      y = 32 * (i % 10);
      enFit[i] = new QCheckBox(s.sprintf("p%i", i), this);
      enFit[i]->setGeometry(x + 10, y + 10, enFit[i]->sizeHint().width(), 22);
      enFit[i]->setChecked(aut->fitPar[i]);
      par[i] = new QLineEdit(this);
      par[i]->setGeometry(x + 55, y + 10, 70, 22);
      par[i]->setText(s.setNum(p[i]));
      l = new QLabel("\261", this);
      l->setGeometry(x + 130, y + 10, 10, 22);
      epar[i] = new QLabel("0", this);
      epar[i]->setFrameStyle(QFrame::Panel | QFrame::Sunken);
      epar[i]->setGeometry(x + 143, y + 10, 70, 22);
    }
    QPushButton *b = new QPushButton(i18n("Load"), this);
    b->setGeometry(10, 345, 85, 25);
    connect(b, SIGNAL(clicked()), SLOT(slotGetParameter()));
    b = new QPushButton(i18n("Save"), this);
    b->setGeometry(105, 345, 85, 25);
    connect(b, SIGNAL(clicked()), SLOT(slotSaveParameter()));
    errCol = new QCheckBox(i18n("Error column"), this);
    errCol->setGeometry(272, 346, 118, 22);
    if (ad->ie < ad->ncols)
      errCol->setChecked(m->aut.fitErrCol);
    else
      errCol->setEnabled(false);
    b = new QPushButton(i18n("Model"), this);
    b->setGeometry(390, 345, 85, 25);
    connect(b, SIGNAL(clicked()), SLOT(slotErrMod()));
    l = new QLabel(i18n("Maximum iterations"), this);
    l->setGeometry(10, 381, 130, 22);
    sItmax = new QSpinBox(1, 200, 1, this);
    sItmax->setGeometry(140, 381, 50, 22);
    sItmax->setValue(aut->fitMaxIt);
    l = new QLabel(i18n("Tolerance"), this);
    l->setGeometry(220, 381, 75, 22);
    eTol = new QLineEdit(this);
    eTol->setGeometry(295, 381, 70, 22);
    eTol->setText(s.setNum(aut->fitTol));
    showCorr = new QPushButton(i18n("Correlations"), this);
    showCorr->setGeometry(390, 380, 85, 25);
    showCorr->setEnabled(false);
    connect(showCorr, SIGNAL(clicked()), SLOT(slotCorr()));
    results = new QListBox(this);
    results->setGeometry(10, 415, 465, 90);
    KSeparator *hl = new KSeparator(KSeparator::HLine, this);
    hl->setGeometry(10, 515, 465, 0);
    b = new QPushButton(i18n("OK"), this);
    b->setGeometry(10, 525, 85, 25);
    b->setDefault(true);
    connect(b, SIGNAL(clicked()), SLOT(slotOK()));
    b = new QPushButton(i18n("Apply"), this);
    b->setGeometry(105, 525, 85, 25);
    connect(b, SIGNAL(clicked()), SLOT(slotApply()));
    b = new QPushButton(i18n("Cancel"), this);
    b->setGeometry(200, 525, 85, 25);
    connect(b, SIGNAL(clicked()), SLOT(reject()));
    b = new QPushButton(i18n("Start"), this);
    b->setGeometry(295, 525, 85, 25);
    connect(b, SIGNAL(clicked()), SLOT(slotFit()));
    b = new QPushButton(i18n("Help"), this);
    b->setGeometry(390, 525, 85, 25);
    connect(b, SIGNAL(clicked()), SLOT(slotHelp()));
    if (m->aut.showSource) {
      QFileInfo fi(fd->path);
      QString path = fi.dirPath(true) + "/" + fi.baseName() + ".c";
      QFile f(path);
      if (f.open(IO_ReadOnly)) {
        QMultiLineEdit *ml = new QMultiLineEdit(this);
        ml->setGeometry(262 * ((NPMAX - 1) / 10) + 233, 10, 500, 540);
        showSource(ml, &f, fd->name);
      }
    }
    adjustSize();
    setFixedSize(size());
  }
}

FitDlg::~FitDlg()
{
  delete aut;
}

void FitDlg::getValues(bool ok)
{
  int i;

  if (dlgMode & SHOWDLG) {
    aut->fitMaxIt = sItmax->value();
    if (ad->ie < ad->ncols)
      aut->fitErrCol = errCol->isChecked();
    for (i = 0; i < NPMAX; i++)
      aut->fitPar[i] = enFit[i]->isChecked();
    if (!getVal(this, eTol, &(aut->fitTol), i18n("invalid tolerance!")))
      return;
    m->aut = *aut;
  }
  for (i = 0; i < NPMAX; i++)
    fd->p[i] = p[i];
  m->setModified();
  emit modelChanged();
  if (ok)
    accept();
}

void FitDlg::saveParameter(QString url)
{
  int i;
  FILE *f;

  KURL u = KURL(url);
  if (u.isLocalFile()) {
    url = u.path();
    QFileInfo fi(url);
    if (fi.exists()) {
      if (KMsgBox::yesNo(this, i18n("Warning"), url + "\n" +
                         i18n("A file with this name exists already.\n"
                         "Do you want to overwrite it?"),
                         KMsgBox::EXCLAMATION) != 1)
        return;
    }
    f = fopen(url, "w");
  } else
    f = fopen(QDir::homeDirPath() + "/kpltmp.tmp", "w");
  if (f) {
    for (i = 0; i < NPMAX; i++)
      fprintf(f, "%.15g %.5g\n", p[i], pErr[i]);
    fclose(f);
    if (u.isLocalFile()) {
      QFileInfo fi(url);
      m->setCurrentDir(fi.dirPath(true));
    } else {
      m->setCurrentDir(u.directoryURL());
      m->kfm = new KFM;
      connect(m->kfm, SIGNAL(finished()), m, SLOT(slotKFM()));
      m->kfm->copy(QDir::homeDirPath() + "/kpltmp.tmp", url);
    }
  } else
    KMsgBox::message(this, i18n("Error"),
                     i18n("while trying to open parameter file"),
                     KMsgBox::EXCLAMATION);
}

void FitDlg::slotMessage(const char *msg)
{
  results->insertItem(msg);
  results->setTopItem(results->count() - 1);
}

void FitDlg::slotGetParameter()
{
  int i;
  QString s;

  getPar(this, p, m);
  for (i = 0; i < NPMAX; i++) {
    par[i]->setText(s.setNum(p[i]));
    pErr[i] = 0.0;
    epar[i]->setText("0");
  }
}

void FitDlg::slotSaveParameter()
{
  QString url = KFileDialog::getSaveFileURL(m->currentDir(), "*.par");
  if (!url.isEmpty())
    saveParameter(url);
}

void FitDlg::slotOK()
{
  getValues(true);
}

void FitDlg::slotApply()
{
  getValues(false);
}

void FitDlg::slotHelp()
{
  kapp->invokeHTMLHelp("kpl/index-5.html", "sec-fit");
}

void FitDlg::slotFit()
{
  int info = 0, i = 0;
  double q = 0.0, avgErr = 0.0;
  const char *infmsg[] = {i18n("Improper input parameters"),
                          i18n("Relative error in chi-square is at most tolerance"),
                          i18n("Relative error in parameters is at most tolerance"),
                          i18n("Relative error in chi-square and in parameters is at most tolerance"),
                          i18n("Function vector is orthogonal to the columns of the jacobian"),
                          i18n("Number of iterations has reached or exceeded maximum"),
                          i18n("Tolerance too small. No further reduction of chi-square possible"),
                          i18n("Tolerance too small. No further improvement of parameters possible")};
  bool valid;
  int j, ij, k, nfev, ny;
  double *wa, *fvec, *diag, *fjac, *qtf, *alpha, *guess, chisq, temp,
         (*fktErrMod)(double, double *);
  void *hErrMod;
  QString s;

  if (!fd->fkt) {
      KMsgBox::message(this, i18n("Error"),
                       s.sprintf(i18n("no function address!"), i),
                       KMsgBox::EXCLAMATION);
      return;
  }
  if (dlgMode & SHOWDLG) {
    if (!getVal(this, eTol, &aut->fitTol, i18n("invalid tolerance!")))
      return;
    aut->fitMaxIt = sItmax->value();
    showCorr->setEnabled(false);
    results->clear();
  }
  np = 0;
  for (i = 0; i < NPMAX; i++) {
    if (dlgMode & SHOWDLG) {
      s = par[i]->text();
      p[i] = s.toDouble(&valid);
      if (!valid) {
        KMsgBox::message(this, i18n("Error"),
                         s.sprintf(i18n("invalid parameter %i"), i),
                         KMsgBox::EXCLAMATION);
        return;
      }
      aut->fitPar[i] = enFit[i]->isChecked();
    }
    if (aut->fitPar[i])
      np++;
  }
  if (!np) {
    KMsgBox::message(this, i18n("Sorry:"), i18n("No parameter to fit!"),
                     KMsgBox::EXCLAMATION);
    return;
  }
  if (!errCol->isChecked())
    if (m->getFun(aut->errModPath, aut->errModName, &hErrMod, &fktErrMod))
      return;
  wa = (double *) calloc((6 + np + ad->n) * np + 2 * ad->n, sizeof(double));
  fvec = wa + 3 * np + ad->n;
  guess = fvec + ad->n;
  diag = guess + np;
  fjac = diag + np;
  qtf = fjac + ad->n * np;
  alpha = qtf + np;
  j = 0;
  for (i = 0; i < NPMAX; i++)
    if (aut->fitPar[i])
      guess[j++] = p[i];
  LMFit lm(this);
  lm.xLM = &ad->x[ad->ix][ad->istart];
  lm.yLM = &ad->x[ad->iy][ad->istart];
  if (errCol->isChecked())
    lm.sig = &ad->x[ad->ie][ad->istart];
  else {
    lm.sig = (double *) malloc(ad->n * sizeof(double));
    for (i = 0; i < ad->n; i++)
       lm.sig[i] = fktErrMod(lm.yLM[i], aut->pErrMod);
    dlclose(hErrMod);
  }
  lm.fLM = fd->fkt;
  lm.bFit = aut->fitPar;
  lm.p = p;
  if (dlgMode & SHOWDLG)
    connect(&lm, SIGNAL(updateMessage(const char *)), SLOT(slotMessage(const char *)));
  lm.lmdif(ad->n, np, guess, fvec, aut->fitTol, aut->fitTol, 0.0,
           (np + 1) * aut->fitMaxIt, 0.0, diag, 1, 100.0, 1, &info, &nfev,
           fjac, ad->n, ip, qtf, wa, wa + np, wa + 2 * np, wa + 3 * np, alpha);
  if (info >= 8)
    info = 4;
  if ((info >= 0) && (dlgMode & SHOWDLG))
    slotMessage(infmsg[info]);
  j = 0;
  for (i = 0; i < NPMAX; i++)
    if (aut->fitPar[i])
      p[i] = guess[j++];
  chisq = lm.enorm(ad->n, fvec);
  chisq *= chisq;
  minv(alpha, corr, np, wa, ip);
  ij = 0;
  for (i = 0; i < np; i++) {
    diag[i] = sqrt(corr[ij]);
    ij += np + 1;
  }
  k = 0;
  for (i = 0; i < np; i++)
    for (j = 0; j < np; j++) {
      temp = diag[i] * diag[j];
      if (temp)
        corr[k] /= temp;
      k++;
    }
  ny = ad->n - np;
  if (ny) {
    q = igamc(0.5 * ny, 0.5 * chisq);
    temp = sqrt(chisq / ny);
    for (i = 0; i < np; i++)
      diag[i] *= temp;
  }
  k = 0;
  for (i = 0; i < NPMAX; i++) {
    if (aut->fitPar[i]) {
      ip[k] = i;
      pErr[i] = diag[k++];
    } else
      pErr[i] = 0.0;
    if (dlgMode & SHOWDLG) {
      par[i]->setText(s.setNum(p[i]));
      epar[i]->setText(s.setNum(pErr[i]));
    }
    if (dlgMode & FOLLOW)
      aut->pFit[i] = p[i];
  }
  for (i = 0; i < ad->n; i++) {
    temp = fvec[i] * lm.sig[i];
    avgErr += temp * temp;
  }
  avgErr = sqrt(avgErr / ad->n);
  if (dlgMode & SHOWDLG) {
    slotMessage(s.sprintf(i18n("Average error = %.3g    ny = %i    Significance Q = %.3g"),
                          avgErr, ny, q));
    showCorr->setEnabled(true);
  }
  if (!errCol->isChecked())
    free(lm.sig);
  free(wa);
  if (dlgMode & SAVEPAR) {
    KURL u = KURL(m->currentFile());
    QFileInfo fi(u.path());
    u.setPath(fi.dirPath(true) + "/" + fi.baseName() + ".par");
    saveParameter(u.url());
  }
}

void FitDlg::slotErrMod(void)
{
  ErrModDlg dlg(this, m, aut);
  dlg.exec();
}

void FitDlg::slotCorr(void)
{
  CorrDlg dlg(this, np, ip, corr);
  dlg.exec();
}
