/***************************************************************************
                          framedlg.h  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sun Apr 25 1999
    copyright            : (C) 1999 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <math.h>
#include <qcheckbox.h>
#include <qspinbox.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qlineedit.h>
#include <kapp.h>
#include <kmsgbox.h>
#include <kcolorbtn.h>
#include "framedlg.h"
#include "kpldoc.h"
#include "edittextdlg.h"
#include "utils.h"

#ifndef min
  #define min(x,y) ((x) < (y) ? (x) : (y))
#endif
#ifndef max
  #define max(x,y) ((x) > (y) ? (x) : (y))
#endif

FrameDlg::FrameDlg(QWidget *parent, KplDoc *model, int it) :
 QDialog(parent, 0, true)
{
  QString s;
  int i;

  m = model;
  item = it;
  pfd = (PFRAMEDATA) m->itd[item].stru;
  fd = *pfd;
  iex = iey = 999;
  setCaption(i18n("Frame"));
  bg = new QButtonGroup(i18n("Axis mode"), this);
  bg->setGeometry(10, 6, 190, 198);
  QRadioButton *r = new QRadioButton(i18n("Frame only"), bg);
  r->setGeometry(12, 32, 175, 24);
  r = new QRadioButton(i18n("Tics"), bg);
  r->setGeometry(12, 64, 175, 24);
  r = new QRadioButton(i18n("Tics with numbers"), bg);
  r->setGeometry(12, 96, 175, 24);
  r = new QRadioButton(i18n("Grid"), bg);
  r->setGeometry(12, 128, 175, 24);
  r = new QRadioButton(i18n("Grid with numbers"), bg);
  r->setGeometry(12, 160, 175, 24);
  r = (QRadioButton *) bg->find(fd.gridmode);
  r->setChecked(true);
  QGroupBox *g = new QGroupBox(i18n("Window"), this);
  g->setGeometry(210, 6, 318, 98);
  QLabel *l = new QLabel(i18n("Left margin"), this);
  l->setGeometry(220, 30, 84, 22);
  ex0 = new QLineEdit(this);
  ex0->setGeometry(304, 30, 50, 22);
  ex0->setText(s.setNum(fd.xw0, 'f', 2));
  l = new QLabel("cm", this);
  l->setGeometry(360, 30, 30, 22);
  l = new QLabel(i18n("Width"), this);
  l->setGeometry(396, 30, 48, 22);
  exl = new QLineEdit(this);
  exl->setGeometry(444, 30, 50, 22);
  exl->setText(s.setNum(fd.xw, 'f', 2));
  l = new QLabel("cm", this);
  l->setGeometry(500, 30, 25, 22);
  l = new QLabel(i18n("Bottom margin"), this);
  l->setGeometry(220, 66, 84, 22);
  ey0 = new QLineEdit(this);
  ey0->setGeometry(304, 66, 50, 22);
  ey0->setText(s.setNum(fd.yw0, 'f', 2));
  l = new QLabel("cm", this);
  l->setGeometry(360, 66, 30, 22);
  l = new QLabel(i18n("Height"), this);
  l->setGeometry(396, 66, 48, 22);
  eyl = new QLineEdit(this);
  eyl->setGeometry(444, 66, 50, 22);
  eyl->setText(s.setNum(fd.yw, 'f', 2));
  l = new QLabel("cm", this);
  l->setGeometry(500, 66, 25, 22);
  g = new QGroupBox(i18n("Scaling"), this);
  g->setGeometry(210, 110, 318, 94);
  l = new QLabel("xmin", this);
  l->setGeometry(220, 134, 40, 22);
  exMin = new QLineEdit(this);
  exMin->setGeometry(260, 134, 80, 22);
  exMin->setText(s.setNum(fd.xmi));
  l = new QLabel("xmax", this);
  l->setGeometry(374, 134, 40, 22);
  exMax = new QLineEdit(this);
  exMax->setGeometry(414, 134, 80, 22);
  exMax->setText(s.setNum(fd.xma));
  l = new QLabel("ymin", this);
  l->setGeometry(220, 168, 40, 22);
  eyMin = new QLineEdit(this);
  eyMin->setGeometry(260, 168, 80, 22);
  eyMin->setText(s.setNum(fd.ymi));
  l = new QLabel("ymax", this);
  l->setGeometry(372, 168, 40, 22);
  eyMax = new QLineEdit(this);
  eyMax->setGeometry(414, 168, 80, 22);
  eyMax->setText(s.setNum(fd.yma));
  g = new QGroupBox(i18n("Colors"), this);
  g->setGeometry(10, 210, 190, 88);
  l = new QLabel(i18n("Frame"), this);
  l->setGeometry(20, 233, 68, 25);
  colFrame = new KColorButton(this, 0);
  colFrame->setGeometry(88, 233, 80, 25);
  colFrame->setColor(QColor(fd.colFrame));
  l = new QLabel(i18n("Grid"), this);
  l->setGeometry(20, 265, 68, 25);
  colGrid = new KColorButton(this, 0);
  colGrid->setGeometry(88, 265, 80, 25);
  colGrid->setColor(QColor(fd.colGrid));
  g = new QGroupBox(i18n("Axes"), this);
  g->setGeometry(210, 210, 318, 88);
  l = new QLabel("xtic", this);
  l->setGeometry(220, 232, 42, 22);
  exTic = new QLineEdit(this);
  exTic->setGeometry(262, 232, 50, 22);
  exTic->setText(s.setNum(fd.xtic));
  l = new QLabel("mticx", this);
  l->setGeometry(328, 232, 40, 22);
  emTicx = new QSpinBox(0, 10, 1, this);
  emTicx->setGeometry(368, 232, 44, 22);
  emTicx->setValue(fd.mticx);
  l = new QLabel("ndigx", this);
  l->setGeometry(426, 232, 44, 22);
  enDigx = new QSpinBox(-1, 8, 1, this);
  enDigx->setGeometry(470, 232, 44, 22);
  enDigx->setValue(fd.ndigx);
  l = new QLabel("ytic", this);
  l->setGeometry(220, 266, 42, 22);
  eyTic = new QLineEdit(this);
  eyTic->setGeometry(262, 266, 50, 22);
  eyTic->setText(s.setNum(fd.ytic));
  l = new QLabel("mticy", this);
  l->setGeometry(328, 266, 40, 22);
  emTicy = new QSpinBox(0, 10, 1, this);
  emTicy->setGeometry(368, 266, 44, 22);
  emTicy->setValue(fd.mticy);
  l = new QLabel("ndigy", this);
  l->setGeometry(426, 266, 44, 22);
  enDigy = new QSpinBox(-1, 8, 1, this);
  enDigy->setGeometry(470, 266, 44, 22);
  enDigy->setValue(fd.ndigy);
  g = new QGroupBox(i18n("Automatic"), this);
  g->setGeometry(10, 304, 190, 88);
  autoNorm = new QCheckBox(i18n("Normalization"), this);
  autoNorm->setGeometry(20, 326, 170, 22);
  autoNorm->setChecked(m->aut.autoNorm);
  QPushButton *b = new QPushButton(i18n("Autoscale"), this);
  b->setGeometry(20, 354, 166, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotAutoScale()));
  bool e = false;
  for (i = item + 1; i < m->nItems; i++) {
    switch (m->itd[i].ityp) {
      case ARRAYITEM:
      case FUNITEM:
      case PARFUNITEM:
        e = true;
        break;
      default:
        break;
    }
    if (m->itd[i].ityp == FRAMEITEM)
      break;
  }
  b->setEnabled(e);
  g = new QGroupBox(i18n("Labels"), this);
  g->setGeometry(210, 304, 318, 124);
  l = new QLabel("iex", this);
  l->setGeometry(220, 326, 26, 22);
  ex = new QSpinBox(-99, 99, 1, this);
  ex->setGeometry(262, 326, 50, 22);
  ex->setValue(fd.iex);
  l = new QLabel(i18n("x axis"), this);
  l->setGeometry(328, 326, 62, 22);
  eLetX = new QLineEdit(this);
  eLetX->setGeometry(390, 326, 90, 22);
  eLetX->setText(fd.sx);
  b = new QPushButton("...", this);
  b->setGeometry(490, 325, 25, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotEditLetX()));
  l = new QLabel("iey", this);
  l->setGeometry(220, 360, 28, 22);
  ey = new QSpinBox(-99, 99, 1, this);
  ey->setGeometry(262, 360, 50, 22);
  ey->setValue(fd.iey);
  l = new QLabel(i18n("y axis"), this);
  l->setGeometry(328, 360, 62, 22);
  eLetY = new QLineEdit(this);
  eLetY->setGeometry(390, 360, 90, 22);
  eLetY->setText(fd.sy);
  b = new QPushButton("...", this);
  b->setGeometry(490, 359, 25, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotEditLetY()));
  l = new QLabel(i18n("Heading"), this);
  l->setGeometry(328, 394, 62, 22);
  eLetH = new QLineEdit(this);
  eLetH->setGeometry(390, 394, 90, 22);
  eLetH->setText(fd.sh);
  b = new QPushButton("...", this);
  b->setGeometry(490, 393, 25, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotEditLetH()));
  l = new QLabel(i18n("Symbol size"), this);
  l->setGeometry(12, 404, 98, 22);
  eRelSiz = new QSpinBox(10, 800, 1, this);
  eRelSiz->setGeometry(114, 404, 72, 22);
  eRelSiz->setValue((int) (100 * fd.relsiz + 0.5));
  eRelSiz->setSuffix(" %");
  b = new QPushButton(i18n("OK"), this);
  b->setGeometry(12, 442, 100, 25);
  b->setDefault(true);
  connect(b, SIGNAL(clicked()), SLOT(slotOK()));
  b = new QPushButton(i18n("Apply"), this);
  b->setGeometry(122, 442, 100, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotApply()));
  b = new QPushButton(i18n("Cancel"), this);
  b->setGeometry(232, 442, 100, 25);
  connect(b, SIGNAL(clicked()), SLOT(reject()));
  b = new QPushButton(i18n("Help"), this);
  b->setGeometry(428, 442, 100, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotHelp()));
  adjustSize();
  setFixedSize(size());
}

void FrameDlg::getValues(bool ok)
{
  int i;
  QString s;
  QRadioButton *rb;

  if (!getVal(this, ex0, &fd.xw0, i18n("invalid position of x!")))
    return;
  if (!getVal(this, ey0, &fd.yw0, i18n("invalid position of y!")))
    return;
  if (!getVal(this, exl, &fd.xw, i18n("invalid width!")))
    return;
  if (!getVal(this, eyl, &fd.yw, i18n("invalid height!")))
    return;
  if (!getVal(this, exMin, &fd.xmi, i18n("invalid xmin!")))
    return;
  if (!getVal(this, exMax, &fd.xma, i18n("invalid xmax!")))
    return;
  if (!getVal(this, eyMin, &fd.ymi, i18n("invalid ymin!")))
    return;
  if (!getVal(this, eyMax, &fd.yma, i18n("invalid ymax!")))
    return;
  if (!getVal(this, exTic, &fd.xtic, i18n("invalid xtic!")))
    return;
  if (!getVal(this, eyTic, &fd.ytic, i18n("invalid ytic!")))
    return;
  fd.colFrame = colFrame->color().rgb();
  fd.colGrid = colGrid->color().rgb();
  fd.iex = ex->value();
  fd.iey = ey->value();
  fd.mticx = emTicx->value();
  fd.mticy = emTicy->value();
  fd.ndigx = enDigx->value();
  fd.ndigy = enDigy->value();
  strcpy(fd.sx, eLetX->text());
  strcpy(fd.sy, eLetY->text());
  strcpy(fd.sh, eLetH->text());
  fd.relsiz = 0.01 * eRelSiz->value();
  fd.gridmode = -1;
  do
    rb = (QRadioButton *) bg->find(++fd.gridmode);
  while (!rb->isChecked());
  if ((iex != 999) && (iey != 999)) {
    for (i = item + 1; i < m->nItems; i++) {
      switch (m->itd[i].ityp) {
        case ARRAYITEM:
          ((PARRAYDATA) m->itd[i].stru)->fx = fx;
          ((PARRAYDATA) m->itd[i].stru)->fy = fy;
          break;
        case FUNITEM:
          ((PFUNDATA) m->itd[i].stru)->fx = fx;
          ((PFUNDATA) m->itd[i].stru)->fy = fy;
          break;
        case PARFUNITEM:
          ((PPARFUNDATA) m->itd[i].stru)->fx = fx;
          ((PPARFUNDATA) m->itd[i].stru)->fy = fy;
          break;
        default:
          break;
      }
      if (m->itd[i].ityp == FRAMEITEM)
        break;
    }
  }
  *pfd = fd;
  m->setModified();
  emit modelChanged();
  if (ok)
    accept();
}

void FrameDlg::editText(QLineEdit *ed)
{
  EditTextDlg dlg(this, ed);
  dlg.exec();
}

void FrameDlg::slotEditLetX()
{
  editText(eLetX);
}

void FrameDlg::slotEditLetY()
{
  editText(eLetY);
}

void FrameDlg::slotEditLetH()
{
  editText(eLetH);
}

void FrameDlg::slotAutoScale()
{
  int i, iext, ieyt, mtic, ndig;
  double xmi, xma, ymi, yma, xtic, ytic, xmin, xmax, ymin, ymax,
         xl, yl, fxt, fyt;
  QString s;

  if (autoNorm->isChecked()) {
    iex = iey = 999;
    for (i = item + 1; i < m->nItems; i++) {
      switch (m->itd[i].ityp) {
        case ARRAYITEM:
          if (((PARRAYDATA) m->itd[i].stru)->x) {
            m->expoFl(&iext, &fxt,
                      &((PARRAYDATA) m->itd[i].stru)->x
                      [((PARRAYDATA) m->itd[i].stru)->ix]
                      [((PARRAYDATA) m->itd[i].stru)->istart],
                      ((PARRAYDATA) m->itd[i].stru)->n);
            m->expoFl(&ieyt, &fyt,
                      &((PARRAYDATA) m->itd[i].stru)->x
                      [((PARRAYDATA) m->itd[i].stru)->iy]
                      [((PARRAYDATA) m->itd[i].stru)->istart],
                      ((PARRAYDATA) m->itd[i].stru)->n);
          }
          break;
        case FUNITEM:
          if (((PFUNDATA) m->itd[i].stru)->fkt) {
            m->expo(max(fabs(((PFUNDATA) m->itd[i].stru)->xfmin),
                    fabs(((PFUNDATA) m->itd[i].stru)->xfmax)),
                    &iext, &fxt);
            m->expoFn(((PFUNDATA) m->itd[i].stru)->fkt, &ieyt, &fyt,
                      ((PFUNDATA) m->itd[i].stru)->xfmin,
                      ((PFUNDATA) m->itd[i].stru)->xfmax,
                      ((PFUNDATA) m->itd[i].stru)->dx,
                      ((PFUNDATA) m->itd[i].stru)->p);
          }
          break;
        case PARFUNITEM:
          if (((PPARFUNDATA) m->itd[i].stru)->fktx &&
              ((PPARFUNDATA) m->itd[i].stru)->fkty) {
            m->expoFn(((PPARFUNDATA) m->itd[i].stru)->fktx, &iext, &fxt,
                      ((PPARFUNDATA) m->itd[i].stru)->tmin,
                      ((PPARFUNDATA) m->itd[i].stru)->tmax,
                      ((PPARFUNDATA) m->itd[i].stru)->dt,
                      ((PPARFUNDATA) m->itd[i].stru)->px);
            m->expoFn(((PPARFUNDATA) m->itd[i].stru)->fkty, &ieyt, &fyt,
                      ((PPARFUNDATA) m->itd[i].stru)->tmin,
                      ((PPARFUNDATA) m->itd[i].stru)->tmax,
                      ((PPARFUNDATA) m->itd[i].stru)->dt,
                      ((PPARFUNDATA) m->itd[i].stru)->py);
          }
          break;
        default:
          break;
      }
      if (m->itd[i].ityp == FRAMEITEM)
        break;
      if (iext < iex) {
        iex = iext;
        fx = fxt;
      }
      if (ieyt < iey) {
        iey = ieyt;
        fy = fyt;
      }
    }
  } else {
    iex = iey = 0;
    fx = fy = 1.0;
  }
  if ((iex != 999) && (iey != 999)) {
    if (!getVal(this, exl, &xl, i18n("invalid width!")))
      return;
    if (!getVal(this, eyl, &yl, i18n("invalid height!")))
      return;
    xmin = ymin = 1.0e300;
    xmax = ymax = -xmin;
    for (i = item + 1; i < m->nItems; i++) {
      switch (m->itd[i].ityp) {
        case ARRAYITEM:
          if (((PARRAYDATA) m->itd[i].stru)->x) {
            m->minMaxFl(&xmi, &xma,
                        &((PARRAYDATA) m->itd[i].stru)->x
                        [((PARRAYDATA) m->itd[i].stru)->ix]
                        [((PARRAYDATA) m->itd[i].stru)->istart],
                        ((PARRAYDATA) m->itd[i].stru)->n);
            m->minMaxFl(&ymi, &yma,
                        &((PARRAYDATA) m->itd[i].stru)->x
                        [((PARRAYDATA) m->itd[i].stru)->iy]
                        [((PARRAYDATA) m->itd[i].stru)->istart],
                        ((PARRAYDATA) m->itd[i].stru)->n);
          }
          break;
        case FUNITEM:
          if (((PFUNDATA) m->itd[i].stru)->fkt) {
            xmi = ((PFUNDATA) m->itd[i].stru)->xfmin;
            xma = ((PFUNDATA) m->itd[i].stru)->xfmax;
            m->autoFn(&ymi, &yma, &ytic, &mtic, &ndig,
                      ((PFUNDATA) m->itd[i].stru)->fkt,
                      xmi, xma,
                      ((PFUNDATA) m->itd[i].stru)->dx,
                      ((PFUNDATA) m->itd[i].stru)->p,
                      1.0, 0.0);
          }
          break;
        case PARFUNITEM:
          if (((PPARFUNDATA) m->itd[i].stru)->fktx &&
              ((PPARFUNDATA) m->itd[i].stru)->fkty) {
            m->autoFn(&xmi, &xma, &xtic, &mtic, &ndig,
                      ((PPARFUNDATA) m->itd[i].stru)->fktx,
                      ((PPARFUNDATA) m->itd[i].stru)->tmin,
                      ((PPARFUNDATA) m->itd[i].stru)->tmax,
                      ((PPARFUNDATA) m->itd[i].stru)->dt,
                      ((PPARFUNDATA) m->itd[i].stru)->px,
                      1.0, 0.0);
            m->autoFn(&ymi, &yma, &ytic, &mtic, &ndig,
                      ((PPARFUNDATA) m->itd[i].stru)->fkty,
                      ((PPARFUNDATA) m->itd[i].stru)->tmin,
                      ((PPARFUNDATA) m->itd[i].stru)->tmax,
                      ((PPARFUNDATA) m->itd[i].stru)->dt,
                      ((PPARFUNDATA) m->itd[i].stru)->py,
                      1.0, 0.0);
          }
          break;
        default:
          break;
      }
      if (m->itd[i].ityp == FRAMEITEM)
        break;
      xmin = min(xmin, xmi);
      xmax = max(xmax, xma);
      ymin = min(ymin, ymi);
      ymax = max(ymax, yma);
    }
    m->autoSc(&xmi, &xma, &xtic, &mtic, &ndig, xmin, xmax, fx, 0.2);
    ex->setValue(iex);
    emTicx->setValue(mtic);
    enDigx->setValue(ndig);
    exMin->setText(s.setNum(xmi));
    exMax->setText(s.setNum(xma));
    exTic->setText(s.setNum(xtic));
    m->autoSc(&ymi, &yma, &ytic, &mtic, &ndig, ymin, ymax, fy,
              0.2 * xl / yl);
    ey->setValue(iey);
    emTicy->setValue(mtic);
    enDigy->setValue(ndig);
    eyMin->setText(s.setNum(ymi));
    eyMax->setText(s.setNum(yma));
    eyTic->setText(s.setNum(ytic));
  }
}

void FrameDlg::slotOK()
{
  getValues(true);
}

void FrameDlg::slotApply()
{
  getValues(false);
}

void FrameDlg::slotHelp()
{
  kapp->invokeHTMLHelp("kpl/index-5.html", "sec-frame");
}
