/***************************************************************************
                          funcdlg.h  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sun Apr 25 1999
    copyright            : (C) 1999 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <dlfcn.h>
#include <qfileinfo.h>
#include <qlistbox.h>
#include <qspinbox.h>
#include <qgroupbox.h>
#include <qlineedit.h>
#include <qmultilinedit.h>
#include <qdir.h>
#include <kapp.h>
#include <kfm.h>
#include <kmsgbox.h>
#include <kfiledialog.h>
#include <kcolorbtn.h>
#include "funcdlg.h"
#include "kpldoc.h"
#include "symboldlg.h"
#include "utils.h"

ChooseFuncDlg::ChooseFuncDlg(QWidget *parent, FILE *f, QLineEdit *func0) :
 QDialog(parent, 0, true)
{
  char s[256];

  func = func0;
  setCaption(i18n("Functions"));
  funcList = new QListBox(this);
  funcList->setGeometry(10, 12, 208, 238);
  while ((fgets(s, 256, f)))
    if (s[9] == 'T')
      funcList->insertItem(strtok(&s[11], "\n"));
  fclose(f);
  connect(funcList, SIGNAL(selected(int)), SLOT(slotList(int)));
  QPushButton *b = new QPushButton(i18n("OK"), this);
  b->setGeometry(8, 264, 100, 25);
  b->setDefault(true);
  connect(b, SIGNAL(clicked()), SLOT(slotOK()));
  b = new QPushButton(i18n("Cancel"), this);
  b->setGeometry(120, 264, 100, 25);
  connect(b, SIGNAL(clicked()), SLOT(reject()));
  adjustSize();
  setFixedSize(size());
}

void ChooseFuncDlg::slotList(int i)
{
  func->setText(funcList->text(i));
  accept();
}

void ChooseFuncDlg::slotOK()
{
  int i;

  if ((i = funcList->currentItem()) != -1) {
    func->setText(funcList->text(i));
    accept();
  }
}

EditParameterDlg::EditParameterDlg(QWidget *parent, double *p0,
                                   const char *file, const char *func) :
 QDialog(parent, 0, true)
{
  QString s;
  QLabel *l;
  int i, x, y;

  p = p0;
  setCaption(i18n("Parameters"));
  for (i = 0; i < NPMAX; i++) {
    x = 128 * (i / 10);
    y = 32 * (i % 10);
    l = new QLabel("p", this);
    l->setGeometry(x + 10, y + 14, 8, 22);
    l = new QLabel(s.setNum(i), this);
    l->setGeometry(x + 18, y + 20, 20, 22);
    par[i] = new QLineEdit(this);
    par[i]->setGeometry(x + 40, y + 14, 70, 22);
    par[i]->setText(s.setNum(p[i]));
  }
  QPushButton *b = new QPushButton(i18n("OK"), this);
  b->setGeometry(18, 338, 100, 25);
  b->setDefault(true);
  connect(b, SIGNAL(clicked()), SLOT(slotOK()));
  b = new QPushButton(i18n("Cancel"), this);
  b->setGeometry(142, 338, 100, 25);
  connect(b, SIGNAL(clicked()), SLOT(reject()));
  if (file) {
    QFileInfo fi(file);
    QString path = fi.dirPath(true) + "/" + fi.baseName() + ".c";
    QFile f(path);
    if (f.open(IO_ReadOnly)) {
      QMultiLineEdit *ml = new QMultiLineEdit(this);
      ml->setGeometry(128 * ((NPMAX - 1) / 10) + 130, 14, 500, 310);
      showSource(ml, &f, func);
    }
  }
  adjustSize();
  setFixedSize(size());
}

void EditParameterDlg::slotOK()
{
  bool valid = true;
  QString s;
  int i;
  double pt[NPMAX];

  for (i = 0; i < NPMAX; i++) {
    s = par[i]->text();
    pt[i] = s.toDouble(&valid);
    if (!valid) {
      KMsgBox::message(this, i18n("Error"),
                       s.sprintf(i18n("invalid parameter %i"), i),
                       KMsgBox::EXCLAMATION);
      return;
    }
  }
  memcpy(p, pt, NPMAX * sizeof(double));
  accept();
}

FuncDlg::FuncDlg(QWidget *parent, KplDoc *model, PFUNDATA fd0) :
 QDialog(parent, 0, true)
{
  QString s;

  m = model;
  fd = fd0;
  fdt = new FUNDATA;
  *fdt = *fd;
  setCaption(i18n("Function"));
  QGroupBox *g = new QGroupBox("y(x)", this);
  g->setGeometry(10, 4, 370, 144);
  QLabel *l = new QLabel(i18n("Library"), this);
  l->setGeometry(20, 30, 90, 22);
  fileName = new QLineEdit(this);
  fileName->setGeometry(110, 30, 225, 22);
  fileName->setText(fd->path);
  QPushButton *b = new QPushButton("...", this);
  b->setGeometry(345, 29, 25, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotFile()));
  l = new QLabel(i18n("Function"), this);
  l->setGeometry(20, 70, 90, 22);
  func = new QLineEdit(this);
  func->setGeometry(110, 70, 225, 22);
  func->setText(fd->name);
  b = new QPushButton("...", this);
  b->setGeometry(345, 69, 25, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotFunc()));
  l = new QLabel(i18n("Parameter"), this);
  l->setGeometry(20, 110, 90, 22);
  b = new QPushButton(i18n("Edit"), this);
  b->setGeometry(110, 109, 80, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotEditParameter()));
  b = new QPushButton(i18n("Load"), this);
  b->setGeometry(200, 109, 80, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotGetParameter()));
  b = new QPushButton(i18n("Save"), this);
  b->setGeometry(290, 109, 80, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotSaveParameter()));
  g = new QGroupBox(this);
  g->setGeometry(10, 154, 370, 176);
  l = new QLabel("x", this);
  l->setGeometry(20, 174, 10, 22);
  l = new QLabel("min", this);
  l->setGeometry(28, 178, 28, 22);
  exMin = new QLineEdit(this);
  exMin->setGeometry(110, 174, 50, 22);
  exMin->setText(s.setNum(fd->xfmin));
  l = new QLabel("x", this);
  l->setGeometry(230, 174, 10, 22);
  l = new QLabel("max", this);
  l->setGeometry(238, 178, 28, 22);
  exMax = new QLineEdit(this);
  exMax->setGeometry(320, 174, 50, 22);
  exMax->setText(s.setNum(fd->xfmax));
  l = new QLabel("D", this);
  l->setGeometry(20, 215, 10, 22);
  QFont f("Symbol");
  l->setFont(f);
  l = new QLabel("x", this);
  l->setGeometry(30, 214, 54, 22);
  exStep = new QLineEdit(this);
  exStep->setGeometry(110, 214, 50, 22);
  exStep->setText(s.setNum(fd->dx));
  b = new QPushButton(i18n("Export"), this);
  b->setGeometry(290, 213, 80, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotExportValues()));
  l = new QLabel(i18n("x normalization"), this);
  l->setGeometry(20, 254, 90, 22);
  efx = new QLineEdit(this);
  efx->setGeometry(110, 254, 50, 22);
  efx->setText(s.setNum(fd->fx));
  l = new QLabel(i18n("y normalization"), this);
  l->setGeometry(230, 254, 90, 22);
  efy = new QLineEdit(this);
  efy->setGeometry(320, 254, 50, 22);
  efy->setText(s.setNum(fd->fy));
  l = new QLabel(i18n("Symbol"), this);
  l->setGeometry(20, 296, 62, 22);
  symb = new QSpinBox(-17, 9, 1, this);
  symb->setGeometry(110, 296, 50, 22);
  symb->setValue(fd->symb);
  b = new QPushButton("...", this);
  b->setGeometry(170, 295, 25, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotSymbol()));
  l = new QLabel(i18n("Color"), this);
  l->setGeometry(230, 296, 90, 22);
  colData = new KColorButton(this, 0);
  colData->setGeometry(320, 295, 50, 25);
  colData->setColor(fd->color);
  b = new QPushButton(i18n("OK"), this);
  b->setGeometry(10, 342, 85, 25);
  b->setDefault(true);
  connect(b, SIGNAL(clicked()), SLOT(slotOK()));
  b = new QPushButton(i18n("Apply"), this);
  b->setGeometry(105, 342, 85, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotApply()));
  b = new QPushButton(i18n("Cancel"), this);
  b->setGeometry(200, 342, 85, 25);
  connect(b, SIGNAL(clicked()), SLOT(reject()));
  b = new QPushButton(i18n("Help"), this);
  b->setGeometry(295, 342, 85, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotHelp()));
  adjustSize();
  setFixedSize(size());
}

FuncDlg::~FuncDlg()
{
  delete fdt;
}

bool FuncDlg::getTemp(PFUNDATA ft)
{
  char *error;

  if (!getVal(this, efx, &fdt->fx, i18n("invalid fx!")))
    return false;
  if (!getVal(this, efy, &fdt->fy, i18n("invalid fy!")))
    return false;
  if (!getVal(this, exMin, &fdt->xfmin, i18n("invalid xmin!")))
    return false;
  if (!getVal(this, exMax, &fdt->xfmax, i18n("invalid xmax!")))
    return false;
  if (!getVal(this, exStep, &fdt->dx, i18n("invalid dx!")))
    return false;
  *ft = *fdt;
  strcpy(ft->path, fileName->text());
  strcpy(ft->name, func->text());
  if (strcmp(fd->path, ft->path)) {
    if (ft->path[0]) {
      if (!(ft->hmod = dlopen(ft->path, RTLD_NOW))) {
        QString msg;
        msg.sprintf(i18n("while trying to open\nfile %s:\n%s"),
                    ft->path, dlerror());
        KMsgBox::message(this, i18n("Error"), msg, KMsgBox::EXCLAMATION);
        return false;
      }
    } else {
      if (ft->name[0]) {
        KMsgBox::message(this, i18n("Error"), i18n("no library specified!"));
        return false;
      }
      ft->hmod = 0;
    }
  }
  if (ft->name[0]) {
    ft->fkt = (double (*)(double, double *)) dlsym(ft->hmod, ft->name);
    if ((error = (char *) dlerror()) != 0) {
      QString msg;
      msg.sprintf(i18n("while trying to determine the address of\nfunction %s:\n%s"),
                  ft->name, error);
      KMsgBox::message(this, i18n("Error"), msg, KMsgBox::EXCLAMATION);
      if (strcmp(fd->path, ft->path))
        dlclose(ft->hmod);
      return false;
    }
  } else
    ft->fkt = 0;
  ft->color = colData->color().rgb();
  ft->symb = symb->value();
  return true;
}

void FuncDlg::getValues(bool ok)
{
  FUNDATA ft;

  if (getTemp(&ft)) {
    if (fd->hmod && (fd->hmod != ft.hmod))
      dlclose(fd->hmod);
    *fd = ft;
    m->setModified();
    emit modelChanged();
    if (ok)
      accept();
  }
}

void FuncDlg::slotFile()
{
  getFile(fileName, m);
}

void FuncDlg::slotFunc()
{
  getFunc(this, fileName, func);
}

void FuncDlg::slotGetParameter()
{
  getPar(this, fdt->p, m);
}

void FuncDlg::slotEditParameter()
{
  if (m->aut.showSource) {
    EditParameterDlg dlg(this, fdt->p, fileName->text(), func->text());
    dlg.exec();
  } else {
    EditParameterDlg dlg(this, fdt->p, 0, 0);
    dlg.exec();
  }
}

void FuncDlg::slotSaveParameter()
{
  savePar(this, fdt->p, m);
}

void FuncDlg::slotExportValues()
{
  double x;
  FILE *f;
  FUNDATA ft;

  if (getTemp(&ft) && ft.fkt) {
    QString url = KFileDialog::getSaveFileURL(m->currentDir(), "*.dat");
    if (!url.isEmpty()) {
      KURL u = KURL(url);
      if (u.isLocalFile()) {
        url = u.path();
        QFileInfo fi(url);
        if (fi.exists())
          if (KMsgBox::yesNo(this, i18n("Warning"),
                             i18n("A file with this name exists already.\n"
                                  "Do you want to overwrite it?"),
                             KMsgBox::EXCLAMATION) != 1)
            return;
        f = fopen(url, "w");
      } else
        f = fopen(QDir::homeDirPath() + "/kpltmp.tmp", "w");
      if (f) {
        if (!ft.dx)
          ft.dx = 0.01 * (ft.xfmax - ft.xfmin);
        x = ft.xfmin;
        fprintf(f, "%g %g\n", ft.fx * x, ft.fy * ((ft.fkt)(x, ft.p)));
        do {
          x += ft.dx;
          if (fabs(x - ft.xfmax) < 0.9 * fabs(ft.dx))
            x = ft.xfmax;
          fprintf(f, "%g %g\n", ft.fx * x, ft.fy * ((ft.fkt)(x, ft.p)));
        } while (x != ft.xfmax);
        fclose(f);
        if (u.isLocalFile()) {
          QFileInfo fi(url);
          m->setCurrentDir(fi.dirPath(true));
        } else {
          m->setCurrentDir(u.directoryURL());
          m->kfm = new KFM;
          m->connectKFM();
          m->kfm->copy(QDir::homeDirPath() + "/kpltmp.tmp", url);
        }
      } else
        KMsgBox::message(this, i18n("Error"),
                         i18n("while trying to open parameter file"), 2);
    }
    if (fd->hmod != ft.hmod)
      dlclose(ft.hmod);
  }
}

void FuncDlg::slotSymbol()
{
  SymbolDlg dlg(this, symb);
  dlg.exec();
}

void FuncDlg::slotOK()
{
  getValues(true);
}

void FuncDlg::slotApply()
{
  getValues(false);
}

void FuncDlg::slotHelp()
{
  kapp->invokeHTMLHelp("kpl/index-5.html", "sec-func");
}

ParFuncDlg::ParFuncDlg(QWidget *parent, KplDoc *model, PPARFUNDATA fd0) :
 QDialog(parent, 0, true)
{
  QString s;

  m = model;
  fd = fd0;
  fdt = new PARFUNDATA;
  *fdt = *fd;
  setCaption(i18n("Parametric function"));
  QGroupBox *g = new QGroupBox("x(t)", this);
  g->setGeometry(10, 4, 370, 144);
  QLabel *l = new QLabel(i18n("Library"), this);
  l->setGeometry(20, 30, 90, 22);
  fileNamex = new QLineEdit(this);
  fileNamex->setGeometry(110, 30, 225, 22);
  fileNamex->setText(fd->pathx);
  QPushButton *b = new QPushButton("...", this);
  b->setGeometry(345, 29, 25, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotFilex()));
  l = new QLabel(i18n("Function"), this);
  l->setGeometry(20, 70, 90, 22);
  funcx = new QLineEdit(this);
  funcx->setGeometry(110, 70, 225, 22);
  funcx->setText(fd->namex);
  b = new QPushButton("...", this);
  b->setGeometry(345, 69, 25, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotFuncx()));
  l = new QLabel(i18n("Parameter"), this);
  l->setGeometry(20, 110, 90, 22);
  b = new QPushButton(i18n("Edit"), this);
  b->setGeometry(110, 109, 80, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotEditParameterx()));
  b = new QPushButton(i18n("Load"), this);
  b->setGeometry(200, 109, 80, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotGetParameterx()));
  b = new QPushButton(i18n("Save"), this);
  b->setGeometry(290, 109, 80, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotSaveParameterx()));
  g = new QGroupBox("y(t)", this);
  g->setGeometry(10, 152, 370, 144);
  l = new QLabel(i18n("Library"), this);
  l->setGeometry(20, 178, 90, 22);
  fileNamey = new QLineEdit(this);
  fileNamey->setGeometry(110, 178, 225, 22);
  fileNamey->setText(fd->pathy);
  b = new QPushButton("...", this);
  b->setGeometry(345, 177, 25, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotFiley()));
  l = new QLabel(i18n("Function"), this);
  l->setGeometry(20, 218, 90, 22);
  funcy = new QLineEdit(this);
  funcy->setGeometry(110, 218, 225, 22);
  funcy->setText(fd->namey);
  b = new QPushButton("...", this);
  b->setGeometry(345, 217, 25, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotFuncy()));
  l = new QLabel(i18n("Parameter"), this);
  l->setGeometry(20, 258, 90, 22);
  b = new QPushButton(i18n("Edit"), this);
  b->setGeometry(110, 257, 80, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotEditParametery()));
  b = new QPushButton(i18n("Load"), this);
  b->setGeometry(200, 257, 80, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotGetParametery()));
  b = new QPushButton(i18n("Save"), this);
  b->setGeometry(290, 257, 80, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotSaveParametery()));
  g = new QGroupBox(this);
  g->setGeometry(10, 304, 370, 176);
  l = new QLabel("t", this);
  l->setGeometry(20, 322, 8, 22);
  l = new QLabel("min", this);
  l->setGeometry(28, 326, 28, 22);
  etMin = new QLineEdit(this);
  etMin->setGeometry(110, 322, 50, 22);
  etMin->setText(s.setNum(fd->tmin));
  l = new QLabel("t", this);
  l->setGeometry(230, 322, 8, 22);
  l = new QLabel("max", this);
  l->setGeometry(238, 326, 28, 22);
  etMax = new QLineEdit(this);
  etMax->setGeometry(320, 322, 50, 22);
  etMax->setText(s.setNum(fd->tmax));
  l = new QLabel("D", this);
  l->setGeometry(20, 363, 10, 22);
  QFont f("Symbol");
  l->setFont(f);
  l = new QLabel("t", this);
  l->setGeometry(30, 362, 54, 22);
  etStep = new QLineEdit(this);
  etStep->setGeometry(110, 362, 50, 22);
  etStep->setText(s.setNum(fd->dt));
  b = new QPushButton(i18n("Export"), this);
  b->setGeometry(290, 361, 80, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotExportValues()));
  l = new QLabel(i18n("x normalization"), this);
  l->setGeometry(20, 402, 90, 22);
  efx = new QLineEdit(this);
  efx->setGeometry(110, 402, 50, 22);
  efx->setText(s.setNum(fd->fx));
  l = new QLabel(i18n("y normalization"), this);
  l->setGeometry(230, 402, 90, 22);
  efy = new QLineEdit(this);
  efy->setGeometry(320, 402, 50, 22);
  efy->setText(s.setNum(fd->fy));
  l = new QLabel(i18n("Symbol"), this);
  l->setGeometry(20, 444, 90, 22);
  symb = new QSpinBox(-17, 9, 1, this);
  symb->setGeometry(110, 444, 50, 22);
  symb->setValue(fd->symb);
  b = new QPushButton("...", this);
  b->setGeometry(174, 443, 25, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotSymbol()));
  l = new QLabel(i18n("Color"), this);
  l->setGeometry(230, 444, 86, 22);
  colData = new KColorButton(this, 0);
  colData->setGeometry(320, 443, 50, 25);
  colData->setColor(fd->color);
  b = new QPushButton(i18n("OK"), this);
  b->setGeometry(10, 490, 85, 25);
  b->setDefault(true);
  connect(b, SIGNAL(clicked()), SLOT(slotOK()));
  b = new QPushButton(i18n("Apply"), this);
  b->setGeometry(105, 490, 85, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotApply()));
  b = new QPushButton(i18n("Cancel"), this);
  b->setGeometry(200, 490, 85, 25);
  connect(b, SIGNAL(clicked()), SLOT(reject()));
  b = new QPushButton(i18n("Help"), this);
  b->setGeometry(295, 490, 85, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotHelp()));
  adjustSize();
  setFixedSize(size());
}

ParFuncDlg::~ParFuncDlg()
{
  delete fdt;
}

bool ParFuncDlg::getTemp(PPARFUNDATA ft)
{
  char *error;

  if (!getVal(this, efx, &fdt->fx, i18n("invalid fx!")))
    return false;
  if (!getVal(this, efy, &fdt->fy, i18n("invalid fy!")))
    return false;
  if (!getVal(this, etMin, &fdt->tmin, i18n("invalid tmin!")))
    return false;
  if (!getVal(this, etMax, &fdt->tmax, i18n("invalid tmax!")))
    return false;
  if (!getVal(this, etStep, &fdt->dt, i18n("invalid dt!")))
    return false;
  *ft = *fdt;
  strcpy(ft->pathx, fileNamex->text());
  strcpy(ft->namex, funcx->text());
  if (strcmp(fd->pathx, ft->pathx)) {
    if (ft->pathx[0]) {
      if (!(ft->hmodx = dlopen(ft->pathx, RTLD_NOW))) {
        QString msg;
        msg.sprintf(i18n("while trying to open\nfile %s:\n%s"),
                    ft->pathx, dlerror());
        KMsgBox::message(this, i18n("Error"), msg, KMsgBox::EXCLAMATION);
        return false;
      }
    } else {
      if (ft->namex[0]) {
        KMsgBox::message(this, i18n("Error"), i18n("no library specified!"));
        return false;
      }
      ft->hmodx = 0;
    }
  }
  if (ft->namex[0]) {
    ft->fktx = (double (*)(double, double *)) dlsym(ft->hmodx, ft->namex);
    if ((error = (char *) dlerror()) != 0) {
      QString msg;
      msg.sprintf(i18n("while trying to determine the address of\nfunction %s:\n%s"),
              ft->namex, error);
      KMsgBox::message(this, i18n("Error"), msg, KMsgBox::EXCLAMATION);
      if (strcmp(fd->pathx, ft->pathx))
        dlclose(ft->hmodx);
      return false;
    }
  } else
    ft->fktx = 0;
  strcpy(ft->pathy, fileNamey->text());
  strcpy(ft->namey, funcy->text());
  if (strcmp(fd->pathy, ft->pathy)) {
    if (ft->pathy[0]) {
      if (!(ft->hmody = dlopen(ft->pathy, RTLD_NOW))) {
        QString msg;
        msg.sprintf(i18n("while trying to open\nfile %s:\n%s"),
                    ft->pathy, dlerror());
        KMsgBox::message(this, i18n("Error"), msg, KMsgBox::EXCLAMATION);
        if (strcmp(fd->pathx, ft->pathx))
          dlclose(ft->hmodx);
        return false;
      }
    } else {
      if (ft->namey[0]) {
        KMsgBox::message(this, i18n("Error"), i18n("no library specified!"));
        if (strcmp(fd->pathx, ft->pathx))
          dlclose(ft->hmodx);
        return false;
      }
      fdt->hmody = 0;
    }
  }
  if (ft->namey[0]) {
    ft->fkty = (double (*)(double, double *)) dlsym(ft->hmody, ft->namey);
    if ((error = (char *) dlerror()) != 0) {
      QString msg;
      msg.sprintf(i18n("while trying to determine the address of\nfunction %s:\n%s"),
                  ft->namey, error);
      KMsgBox::message(this, i18n("Error"), msg, KMsgBox::EXCLAMATION);
      if (strcmp(fd->pathx, ft->pathx))
        dlclose(ft->hmodx);
      return false;
    }
  } else
    ft->fkty = 0;
  ft->color = colData->color().rgb();
  ft->symb = symb->value();
  return true;
}

void ParFuncDlg::getValues(bool ok)
{
  PARFUNDATA ft;

  if (getTemp(&ft)) {
    if (fd->hmodx && (fd->hmodx != ft.hmodx))
      dlclose(fd->hmodx);
    if (fd->hmody && (fd->hmody != ft.hmody))
      dlclose(fd->hmody);
    *fd = ft;
    m->setModified();
    emit modelChanged();
    if (ok)
      accept();
  }
}

void ParFuncDlg::slotFilex()
{
  getFile(fileNamex, m);
}

void ParFuncDlg::slotFiley()
{
  getFile(fileNamey, m);
}

void ParFuncDlg::slotFuncx()
{
  getFunc(this, fileNamex, funcx);
}

void ParFuncDlg::slotFuncy()
{
  getFunc(this, fileNamey, funcy);
}

void ParFuncDlg::slotGetParameterx()
{
  getPar(this, fdt->px, m);
}

void ParFuncDlg::slotGetParametery()
{
  getPar(this, fdt->py, m);
}

void ParFuncDlg::slotEditParameterx()
{
  if (m->aut.showSource) {
    EditParameterDlg dlg(this, fdt->px, fileNamex->text(), funcx->text());
    dlg.exec();
  } else {
    EditParameterDlg dlg(this, fdt->px, 0, 0);
    dlg.exec();
  }
}

void ParFuncDlg::slotEditParametery()
{
  if (m->aut.showSource) {
    EditParameterDlg dlg(this, fdt->py, fileNamey->text(), funcy->text());
    dlg.exec();
  } else {
    EditParameterDlg dlg(this, fdt->py, 0, 0);
    dlg.exec();
  }
}

void ParFuncDlg::slotSaveParameterx()
{
  savePar(this, fdt->px, m);
}

void ParFuncDlg::slotSaveParametery()
{
  savePar(this, fdt->py, m);
}

void ParFuncDlg::slotExportValues()
{
  double t;
  FILE *f;
  PARFUNDATA ft;

  if (getTemp(&ft) && ft.fktx && ft.fkty) {
    QString url = KFileDialog::getSaveFileURL(m->currentDir(), "*.dat");
    if (!url.isEmpty()) {
      KURL u = KURL(url);
      if (u.isLocalFile()) {
        url = u.path();
        QFileInfo fi(url);
        if (fi.exists())
          if (KMsgBox::yesNo(this, i18n("Warning"),
                             i18n("A file with this name exists already.\n"
                                  "Do you want to overwrite it?"),
                             KMsgBox::EXCLAMATION) != 1)
            return;
        f = fopen(url, "w");
      } else
        f = fopen(QDir::homeDirPath() + "/kpltmp.tmp", "w");
      if (f) {
        if (!ft.dt)
          ft.dt = 0.01 * (ft.tmax - ft.tmin);
        t = ft.tmin;
        fprintf(f, "%g %g\n", ft.fx * ((ft.fktx)(t, ft.px)),
                ft.fy * ((ft.fkty)(t, ft.py)));
        do {
          t += ft.dt;
          if (fabs(t - ft.tmax) < 0.9 * fabs(ft.dt))
            t = ft.tmax;
          fprintf(f, "%g %g\n", ft.fx * ((ft.fktx)(t, ft.px)),
                  ft.fy * ((ft.fkty)(t, ft.py)));
        } while (t != ft.tmax);
        fclose(f);
        if (u.isLocalFile()) {
          QFileInfo fi(url);
          m->setCurrentDir(fi.dirPath(true));
        } else {
          m->setCurrentDir(u.directoryURL());
          m->kfm = new KFM;
          m->connectKFM();
          m->kfm->copy(QDir::homeDirPath() + "/kpltmp.tmp", url);
        }
      } else
        KMsgBox::message(this, i18n("Error"),
                         i18n("while trying to open parameter file"), 2);
    }
    if (fd->hmodx != ft.hmodx)
      dlclose(ft.hmodx);
    if (fd->hmody != ft.hmody)
      dlclose(ft.hmody);
  }
}

void ParFuncDlg::slotSymbol()
{
  SymbolDlg dlg(this, symb);
  dlg.exec();
}

void ParFuncDlg::slotOK()
{
  getValues(true);
}

void ParFuncDlg::slotApply()
{
  getValues(false);
}

void ParFuncDlg::slotHelp()
{
  kapp->invokeHTMLHelp("kpl/index-5.html", "sec-parfunc");
}
