/***************************************************************************
                          itemdlg.cpp  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sun Apr 25 1999
    copyright            : (C) 1999 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <dlfcn.h>
#include <qfileinfo.h>
#include <qradiobutton.h>
#include <qpushbutton.h>
#include <qbuttongroup.h>
#include <qlistview.h>
#include <kapp.h>
#include "itemdlg.h"
#include "kpldoc.h"
#include "framedlg.h"
#include "arraydlg.h"
#include "funcdlg.h"
#include "legenddlg.h"
#include "fitdlg.h"

NewItemDlg::NewItemDlg(QWidget *parent, int *i) :
 QDialog(parent, 0, true)
{
  ityp = i;
  setCaption(i18n("New item"));
  bg = new QButtonGroup();
  QRadioButton *r = groupButton(i18n("Frame"), 14, 22, 160, 24);
  r->setChecked(true);
  r = groupButton(i18n("Array"), 14, 54, 160, 24);
  r = groupButton(i18n("Function"), 14, 86, 160, 24);
  r = groupButton(i18n("Parametric Function"), 14, 118, 160, 24);
  r = groupButton(i18n("Legend"), 14, 150, 160, 24);
  QPushButton *b = new QPushButton(i18n("OK"), this);
  b->setGeometry(48, 192, 100, 25);
  b->setDefault(true);
  connect(b, SIGNAL(clicked()), SLOT(slotOK()));
  adjustSize();
  setFixedSize(size());
}

QRadioButton *NewItemDlg::groupButton(const char *text,
                                      int x, int y, int w, int h)
{
  QRadioButton *b;

  b = new QRadioButton(text, this);
  b->setGeometry(x, y, w, h);
  bg->insert(b);
  return b;
}

void NewItemDlg::slotOK()
{
  QRadioButton *rb;

  *ityp = 0;
  do 
    rb = (QRadioButton *) bg->find((*ityp)++);
  while (!rb->isChecked());
  accept();
}

ItemDlg::ItemDlg(QWidget *parent, KplDoc *model) :
 QDialog(parent, 0, true)
{
  m = model;
  change = false;
  iarr = ifun = -1;
  lItem = 0;
  setCaption(i18n("Items"));
  popupMenu = new QPopupMenu();
  popupMenu->insertItem(i18n("&Edit"), this, SLOT(slotEdit()));
  popupMenu->insertItem(i18n("&Delete"), this, SLOT(slotDelete()));
  popupMenu->insertItem(i18n("&Copy"), this, SLOT(slotCopy()));
  itemList = new QListView(this);
  itemList->setGeometry(8, 12, 378, 241);
  itemList->addColumn(i18n("Type"));
  itemList->addColumn(i18n("Settings"));
  itemList->setRootIsDecorated(true);
  itemList->setColumnWidthMode(0, QListView::Manual);
  itemList->setColumnWidth(0, 120);
  itemList->setColumnWidthMode(1, QListView::Manual);
  itemList->setColumnWidth(1, 258);
  itemList->setAllColumnsShowFocus(true);
  connect(itemList, SIGNAL(doubleClicked(QListViewItem *)),
          SLOT(slotList(QListViewItem *)));
  connect(itemList, SIGNAL(returnPressed(QListViewItem *)),
          SLOT(slotList(QListViewItem *)));
  connect(itemList, SIGNAL(rightButtonClicked(QListViewItem *, const QPoint &, int)),
          SLOT(slotPopup(QListViewItem *, const QPoint &, int)));
  QPushButton *b = new QPushButton(i18n("New"), this);
  b->setGeometry(396, 12, 90, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotNew()));
  edit = new QPushButton(i18n("Edit"), this);
  edit->setGeometry(396, 48, 90, 25);
  connect(edit, SIGNAL(clicked()), SLOT(slotEdit()));
  fit = new QPushButton(i18n("Fit"), this);
  fit->setGeometry(396, 84, 90, 25);
  connect(fit, SIGNAL(clicked()), SLOT(slotFit()));
  del = new QPushButton(i18n("Delete"), this);
  del->setGeometry(396, 120, 90, 25);
  connect(del, SIGNAL(clicked()), SLOT(slotDelete()));
  copy = new QPushButton(i18n("Copy"), this);
  copy->setGeometry(396, 156, 90, 25);
  connect(copy, SIGNAL(clicked()), SLOT(slotCopy()));
  b = new QPushButton(i18n("Help"), this);
  b->setGeometry(396, 192, 90, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotHelp()));
  b = new QPushButton(i18n("Close"), this);
  b->setGeometry(396, 228, 90, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotClose()));
  adjustSize();
  setFixedSize(size());
  updItemList();
}

void ItemDlg::updItemList()
{
  int i;
  QString s, s2;
  QFileInfo fi;
  QString fname;
  QListViewItem *parent = 0;

  nFuncs = nArrays = 0;
  itemList->clear();
  itemList->setSorting(-1);
  if (lItem) {
    free(lItem);
    lItem = 0;
  }
  if (m->nItems) {
    lItem = (QListViewItem **) malloc(m->nItems * sizeof(QListViewItem *));
    for (i = 0; i < m->nItems; i++) {
      if (m->itd[i].ityp == FRAMEITEM) {
        if (parent)
          parent = lItem[i] = new QListViewItem(itemList, parent);
        else
          parent = lItem[i] = new QListViewItem(itemList);
        parent->setOpen(true);
      } else
        if (parent)
          lItem[i] = new QListViewItem(parent, lItem[i - 1]);
        else
          if (i)
            lItem[i] = new QListViewItem(itemList, lItem[i - 1]);
          else
            lItem[i] = new QListViewItem(itemList);
      switch (m->itd[i].ityp) {
        case FRAMEITEM:
          lItem[i]->setText(0, i18n("Frame"));
          lItem[i]->setText(1, s.sprintf(i18n("x = %g cm, y = %g cm, size %g cm x %g cm"),
                            ((PFRAMEDATA) m->itd[i].stru)->xw0,
                            ((PFRAMEDATA) m->itd[i].stru)->yw0,
                            ((PFRAMEDATA) m->itd[i].stru)->xw,
                            ((PFRAMEDATA) m->itd[i].stru)->yw));
          break;
        case ARRAYITEM:
          lItem[i]->setText(0, i18n("Array"));
          fi.setFile(((PARRAYDATA) m->itd[i].stru)->path);
          fname = fi.fileName();
          lItem[i]->setText(1, s.sprintf(i18n("file %s, col. %2i %2i"),
                            fname.data(),
                            ((PARRAYDATA) m->itd[i].stru)->ix,
                            ((PARRAYDATA) m->itd[i].stru)->iy));
          nArrays++;
          break;
        case FUNITEM:
          lItem[i]->setText(0, i18n("Function"));
          fi.setFile(((PFUNDATA) m->itd[i].stru)->path);
          fname = fi.baseName();
          lItem[i]->setText(1, s.sprintf("%s.%s", fname.data(),
                            ((PFUNDATA) m->itd[i].stru)->name));
          nFuncs++;
          break;
        case PARFUNITEM:
          lItem[i]->setText(0, i18n("Par. function"));
          fi.setFile(((PPARFUNDATA) m->itd[i].stru)->pathx);
          fname = fi.baseName();
          s2.sprintf("x %s.%s", fname.data(),
                     ((PPARFUNDATA) m->itd[i].stru)->namex);
          fi.setFile(((PPARFUNDATA) m->itd[i].stru)->pathy);
          fname = fi.baseName();
          lItem[i]->setText(1, s.sprintf("%s, y %s.%s", (const char *) s2,
                            fname.data(),
                            ((PPARFUNDATA) m->itd[i].stru)->namey));
          break;
        case LEGENDITEM:
          lItem[i]->setText(0, i18n("Legend"));
          lItem[i]->setText(1, s.sprintf(i18n("symbol %2i"),
                            ((PLEGENDDATA) m->itd[i].stru)->symb));
      }
    }
    itemList->setCurrentItem(lItem[0]);
  }
  edit->setEnabled(m->nItems);
  fit->setEnabled(nArrays && nFuncs);
  copy->setEnabled(m->nItems);
  del->setEnabled(m->nItems);
  itemList->show();
}

int ItemDlg::editItem(int i)
{
  switch (m->itd[i].ityp) {
    case FRAMEITEM:
      {
        FrameDlg dlg(this, m, i);
        connect(&dlg, SIGNAL(modelChanged()), m, SIGNAL(modelChanged()));
        return dlg.exec();
      }
    case ARRAYITEM:
      {
        ArrayDlg dlg(this, m, (PARRAYDATA) m->itd[i].stru);
        connect(&dlg, SIGNAL(modelChanged()), m, SIGNAL(modelChanged()));
        return dlg.exec();
      }
    case FUNITEM:
      {
        FuncDlg dlg(this, m, (PFUNDATA) m->itd[i].stru);
        connect(&dlg, SIGNAL(modelChanged()), m, SIGNAL(modelChanged()));
        return dlg.exec();
      }
    case PARFUNITEM:
      {
        ParFuncDlg dlg(this, m, (PPARFUNDATA) m->itd[i].stru);
        connect(&dlg, SIGNAL(modelChanged()), m, SIGNAL(modelChanged()));
        return dlg.exec();
      }
    case LEGENDITEM:
      {
        LegendDlg dlg(this, m, (PLEGENDDATA) m->itd[i].stru);
        connect(&dlg, SIGNAL(modelChanged()), m, SIGNAL(modelChanged()));
        return dlg.exec();
      }
  }
  return 0;
}

void ItemDlg::copyStru(int id, int is)
{
  m->itd[id] = m->itd[is];
  m->itd[id].stru = malloc(m->iSize[m->itd[id].ityp]);
  memcpy(m->itd[id].stru, m->itd[is].stru, m->iSize[m->itd[is].ityp]);
  switch (m->itd[id].ityp) {
    case FUNITEM:
      if (((PFUNDATA) m->itd[id].stru)->hmod)
        ((PFUNDATA) m->itd[id].stru)->hmod =
          dlopen(((PFUNDATA) m->itd[id].stru)->path, RTLD_NOW);
      break;
    case PARFUNITEM:
      if (((PPARFUNDATA) m->itd[id].stru)->hmodx)
        ((PPARFUNDATA) m->itd[id].stru)->hmodx =
          dlopen(((PPARFUNDATA) m->itd[id].stru)->pathx, RTLD_NOW);
      if (((PPARFUNDATA) m->itd[id].stru)->hmody)
        ((PPARFUNDATA) m->itd[id].stru)->hmody =
          dlopen(((PPARFUNDATA) m->itd[id].stru)->pathy, RTLD_NOW);
      break;
    default:
      break;
  }
}

int ItemDlg::indexItem(QListViewItem *it)
{
  int i;

  if (lItem) {
    for (i = 0; i < m->nItems; i++)
      if (lItem[i] == it)
        return i;
  }
  return -1;
}

void ItemDlg::slotList(QListViewItem *i)
{
  if (editItem(indexItem(i)))
    change = true;
  updItemList();
}

void ItemDlg::slotNew()
{
  int i;

  (m->nItems)++;
  m->itd = (PITEMDATA) realloc(m->itd, m->nItems * sizeof(ITEMDATA));
  NewItemDlg dlg(this, &i);
  dlg.exec();
  m->iniStru(&(m->itd[m->nItems - 1]), (itemTypes) i);
  editItem(m->nItems - 1);
  updItemList();
  change = true;
}

void ItemDlg::slotEdit()
{
  QListViewItem *i;

  if (!itemList->isMultiSelection()) {
    if ((i = itemList->currentItem()))
      slotList(i);
  } else
    itemList->setMultiSelection(false);
}

void ItemDlg::slotFit()
{
  int i;

  if (itemList->isMultiSelection()) {
    int n = 0;
    for (i = 0; i < m->nItems; i++)
      if (itemList->isSelected(lItem[i])) {
        if (m->itd[i].ityp == ARRAYITEM)
          iarr = i;
        if (m->itd[i].ityp == FUNITEM)
          ifun = i;
        n++;
      }
    if ((n != 2) || (ifun < 0) || (iarr < 0))
      return;
  } else {
    itemList->setMultiSelection(true);
    if ((iarr >= 0) && (iarr < m->nItems) &&
        (ifun >= 0) && (ifun < m->nItems)) {
      itemList->setSelected(lItem[iarr], true);
      itemList->setSelected(lItem[ifun], true);
    }
    emit displayMessage(i18n("Select one data set and one function!"));
    return;
  }
  emit displayMessage(i18n("Fitting function parameters..."));
  FitDlg dlg(this, m, (PARRAYDATA) m->itd[iarr].stru,
             (PFUNDATA) m->itd[ifun].stru, SHOWDLG);
  connect(&dlg, SIGNAL(modelChanged()), m, SIGNAL(modelChanged()));
  if (dlg.exec())
    change = true;
  updItemList();
  itemList->setMultiSelection(false);
  emit displayMessage(i18n("Editing plot items..."));
}

void ItemDlg::slotDelete()
{
  int i,j;

  if (!itemList->isMultiSelection()) {
    if (itemList->currentItem()) {
      i = indexItem(itemList->currentItem());
      m->freeItd(&(m->itd[i]));
      m->nItems--;
      for (j = i; j < m->nItems; j++)
        m->itd[j] = m->itd[j + 1];
        updItemList();
        change = true;
    }
  } else
    itemList->setMultiSelection(false);
}

void ItemDlg::slotCopy()
{
  if (!itemList->isMultiSelection()) {
    if (itemList->currentItem()) {
      int i = indexItem(itemList->currentItem());
      m->nItems++;
      m->itd = (PITEMDATA) realloc(m->itd, m->nItems * sizeof(ITEMDATA));
      copyStru(m->nItems - 1, i);
      updItemList();
      change = true;
    }
  } else
    itemList->setMultiSelection(false);
}

void ItemDlg::slotClose()
{
  done(change);
}

void ItemDlg::slotHelp()
{
  kapp->invokeHTMLHelp("kpl/index-5.html", "sec-items");
}

void ItemDlg::slotPopup(QListViewItem *item, const QPoint &p, int)
{
  if (item) {
    itemList->setCurrentItem(item);
    itemList->setSelected(item, true);
    popupMenu->exec(p);
  }
}
