/***************************************************************************
                          itemdlg.h  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sun Apr 25 1999
    copyright            : (C) 1999 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ITEMDLG_H
#define ITEMDLG_H

#include <qdialog.h> 

class QListView;
class QListViewItem;
class QButtonGroup;
class QRadioButton;
class QPopupMenu;
class KplDoc;

/**
  * New item  dialog class. Provides a dialog for the selection of the type
  * of a new plot item.
  * @author Werner Stille
  */
class NewItemDlg : public QDialog {
  Q_OBJECT

public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param i pointer to item type.
   */
  NewItemDlg(QWidget *parent, int *i);
  /**
   * Creates a new groupbutton.
   * @param text button label.
   * @param x horizontal position.
   * @param y vertical position.
   * @param w width.
   * @param h height.
   */
  QRadioButton *groupButton(const char *text,
                            int x, int y, int w, int h);

protected slots:
  /** Accepts selection and leaves the dialog. */
  void slotOK();

private:
  int *ityp;
  QButtonGroup *bg;
};

/**
  * Items dialog class. Provides a dialog for editing the plot item list.
  * @author Werner Stille
  */
class ItemDlg : public QDialog {
  Q_OBJECT

public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param model pointer to KplDoc model object.
   */
  ItemDlg(QWidget *parent, KplDoc *model);

protected:
  /** Updates the item list. */
  void updItemList();
  /**
   * Displays a dialog to edit an item according to its type.
   * @param i index of selected item.
   */
  int editItem(int i);
  /**
   * Append a copy of the selected item.
   * @param id destination item index.
   * @param is source item index.
   */
  void copyStru(int id, int is);
  /**
   * Returns index of list item.
   * @param it list item.
   * @return item index.
   */
  int indexItem(QListViewItem *it);

protected slots:
  /**
   * Displays a dialog to edit an item according to its type.
   * @param i selected item.
   */
  void slotList(QListViewItem *i);
  /** Displays the dialog to choose the type of a new item. */
  void slotNew();
  /** Displays a dialog to edit an item according to its type. */
  void slotEdit();
  /** Enters parameter fitting. */
  void slotFit();
  /** Deletes the selected item. */
  void slotDelete();
  /** Append a copy of the selected item at the end of the list. */
  void slotCopy();
  /** Applies settings and leaves the dialog. */
  void slotClose();
  /** Displays help for the function settings dialog. */
  void slotHelp();
  /**
   * Displays a popup menu.
   * @param item pointer to clicked QListViewItem.
   * @param point where popup menu will appear.
   */
  void slotPopup(QListViewItem *item, const QPoint &p, int);

signals:
  /**
   * Emitted when status message should be displayed.
   * @param msg message text.
   */
  void displayMessage(const char *msg);

private:
  KplDoc *m;
  bool change;
  int nArrays, nFuncs, iarr, ifun;
  QListView *itemList;
  QListViewItem **lItem;
  QPushButton *edit, *fit, *del, *copy;
  QPopupMenu *popupMenu;
};

#endif

