/***************************************************************************
                          kplots.h  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sun Apr 25 1999
    copyright            : (C) 1999 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KPLOTS_H
#define KPLOTS_H

#define NPMAX 20

/** Types of files */
enum fileTypes{UNKNOWN, DATAFILE, PLOTFILE};
/** Types of items */
enum itemTypes{FRAMEITEM = 1, ARRAYITEM, FUNITEM, PARFUNITEM, LEGENDITEM};

typedef struct {
  int ndigx, ndigy, mticx, mticy, gridmode, iex, iey;
  long colFrame, colGrid;
  double xf, yf, xw0, xw, yw0, yw, xmi, xma, ymi, yma, xtic, ytic, relsiz;
  char sx[80], sy[80], sh[260];
} FRAMEDATA;
typedef FRAMEDATA *PFRAMEDATA;

typedef struct {
  int ix, iy, ie, istart, n, symb, errbars, nrows, ncols;
  long color;
  double fx, fy, **x;
  char path[260];
} ARRAYDATA;
typedef ARRAYDATA *PARRAYDATA;

typedef struct {
  int symb;
  long color;
  double xfmin, xfmax, dx, fx, fy, (*fkt)(double, double *), p[NPMAX];
  char name[100], path[260];
  void *hmod;
} FUNDATA;
typedef FUNDATA *PFUNDATA;

typedef struct {
  int symb;
  long color;
  double tmin, tmax, dt, fx, fy,
         (*fktx)(double, double *), px[NPMAX],
         (*fkty)(double, double *), py[NPMAX];
  char namex[100], namey[100], pathx[260], pathy[260];
  void *hmodx, *hmody;
} PARFUNDATA;
typedef PARFUNDATA *PPARFUNDATA;

typedef struct {
  int symb;
  long colsymb, coltext;
  double x, y, xtext, ytext, relsiz;
  char s[80];
} LEGENDDATA;
typedef LEGENDDATA *PLEGENDDATA;

typedef struct {
  itemTypes ityp;
  void *stru;
} ITEMDATA;
typedef ITEMDATA *PITEMDATA;

typedef struct {
  double xf, yf;
  long colFrame, colGrid, colData;
} BasicStruct;

typedef struct {
  bool autoErr, autoNorm, autohPath, autoFit, fitErrCol, fitShowDlg,
       fitSavePar, fitFollow, fitPar[NPMAX], showSource;
  int ixAuto, iyAuto, ieAuto, autoGrid, autoSymb, fitMaxIt, fitSymb;
  long fitColor;
  double x0Auto, xlAuto, y0Auto, ylAuto, fitTol,
         pFit[NPMAX], pErrMod[NPMAX];
  char sxAuto[80], syAuto[80], shAuto[260], fitName[100], fitPath[260],
       errModName[100], errModPath[260];
} AutoStruct;

#endif
