/***************************************************************************
                          legenddlg.h  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sun Apr 25 1999
    copyright            : (C) 1999 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qlineedit.h>
#include <qspinbox.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <kapp.h>
#include <kcolorbtn.h>
#include "legenddlg.h"
#include "kpldoc.h"
#include "edittextdlg.h"
#include "symboldlg.h"
#include "utils.h"

LegendDlg::LegendDlg(QWidget *parent, KplDoc *model, PLEGENDDATA ld0) :
 QDialog(parent, 0, true)
{
  QString s;

  m = model;
  ld = ld0;
  setCaption(i18n("Legend"));
  QGroupBox *g = new QGroupBox(i18n("Data"), this);
  g->setGeometry(10, 6, 370, 94);
  QLabel *l = new QLabel(i18n("Symbol"), this);
  l->setGeometry(20, 34, 60, 22);
  symb = new QSpinBox(-17, 9, 1, this);
  symb->setGeometry(90, 34, 60, 22);
  symb->setValue(ld->symb);
  QPushButton *b = new QPushButton("...", this);
  b->setGeometry(160, 33, 25, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotSymbol()));
  l = new QLabel(i18n("Color"), this);
  l->setGeometry(218, 32, 54, 25);
  colSymb = new KColorButton(this, 0);
  colSymb->setGeometry(272, 32, 60, 25);
  colSymb->setColor(ld->colsymb);
  l = new QLabel("x", this);
  l->setGeometry(20, 64, 58, 22);
  ex0 = new QLineEdit(this);
  ex0->setGeometry(90, 64, 60, 22);
  ex0->setText(s.setNum(ld->x, 'f', 2));
  l = new QLabel("cm", this);
  l->setGeometry(158, 64, 26, 22);
  l = new QLabel("y", this);
  l->setGeometry(218, 64, 58, 22);
  ey0 = new QLineEdit(this);
  ey0->setGeometry(272, 64, 60, 22);
  ey0->setText(s.setNum(ld->y, 'f', 2));
  l = new QLabel("cm", this);
  l->setGeometry(338, 64, 26, 22);
  g = new QGroupBox(i18n("Label"), this);
  g->setGeometry(10, 106, 370, 134);
  l = new QLabel(i18n("Text"), this);
  l->setGeometry(20, 132, 58, 22);
  eText = new QLineEdit(this);
  eText->setGeometry(90, 132, 242, 22);
  eText->setText(ld->s);
  b = new QPushButton("...", this);
  b->setGeometry(346, 132, 25, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotEditText()));
  l = new QLabel(i18n("Size"), this);
  l->setGeometry(20, 168, 70, 22);
  eRelSize = new QSpinBox(10, 800, 1, this);
  eRelSize->setGeometry(90, 168, 60, 22);
  eRelSize->setValue((int) (100 * ld->relsiz + 0.5));
  eRelSize->setSuffix(" %");
  l = new QLabel(i18n("Color"), this);
  l->setGeometry(218, 167, 54, 25);
  colText = new KColorButton(this, 0);
  colText->setGeometry(272, 167, 60, 25);
  colText->setColor(ld->coltext);
  l = new QLabel("x", this);
  l->setGeometry(20, 204, 8, 22);
  l = new QLabel("off", this);
  l->setGeometry(28, 208, 50, 22);
  exl = new QLineEdit(this);
  exl->setGeometry(90, 204, 60, 22);
  exl->setText(s.setNum(ld->xtext, 'f', 2));
  l = new QLabel("cm", this);
  l->setGeometry(158, 204, 26, 22);
  l = new QLabel("y", this);
  l->setGeometry(218, 204, 8, 22);
  l = new QLabel("off", this);
  l->setGeometry(226, 208, 46, 22);
  eyl = new QLineEdit(this);
  eyl->setGeometry(272, 204, 60, 22);
  eyl->setText(s.setNum(ld->ytext, 'f', 2));
  l = new QLabel("cm", this);
  l->setGeometry(338, 204, 26, 22);
  b = new QPushButton(i18n("OK"), this);
  b->setGeometry(10, 252, 85, 25);
  b->setDefault(true);
  connect(b, SIGNAL(clicked()), SLOT(slotOK()));
  b = new QPushButton(i18n("Apply"), this);
  b->setGeometry(105, 252, 85, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotApply()));
  b = new QPushButton(i18n("Cancel"), this);
  b->setGeometry(200, 252, 85, 25);
  connect(b, SIGNAL(clicked()), SLOT(reject()));
  b = new QPushButton(i18n("Help"), this);
  b->setGeometry(295, 252, 85, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotHelp()));
  adjustSize();
  setFixedSize(size());
}

void LegendDlg::getValues(bool ok)
{
  double x, y, xtext, ytext;

  if (!getVal(this, ex0, &x, i18n("invalid x position!")))
    return;
  if (!getVal(this, ey0, &y, i18n("invalid y position!")))
    return;
  if (!getVal(this, exl, &xtext, i18n("invalid x position for text!")))
    return;
  if (!getVal(this, eyl, &ytext, i18n("invalid y position for text!")))
    return;
  ld->x = x;
  ld->y = y;
  ld->xtext = xtext;
  ld->ytext = ytext;
  ld->colsymb = colSymb->color().rgb();
  ld->coltext = colText->color().rgb();
  ld->symb = symb->value();
  ld->relsiz = 0.01 * eRelSize->value();
  strcpy(ld->s, eText->text());
  m->setModified();
  emit modelChanged();
  if (ok)
    accept();
}

void LegendDlg::slotEditText()
{
  EditTextDlg dlg(this, eText);
  dlg.exec();
}

void LegendDlg::slotSymbol()
{
  SymbolDlg dlg(this, symb);
  dlg.exec();
}

void LegendDlg::slotOK()
{
  getValues(true);
}

void LegendDlg::slotApply()
{
  getValues(false);
}

void LegendDlg::slotHelp()
{
  kapp->invokeHTMLHelp("kpl/index-5.html", "sec-legend");
}
