/***************************************************************************
                          legenddlg.h  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sun Apr 25 1999
    copyright            : (C) 1999 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef LEGENDDLG_H
#define LEGENDDLG_H

#include <qdialog.h>
#include "kplots.h"

class KplDoc;
class QLineEdit;
class QSpinBox;
class KColorButton;

/**
  * Legend settings dialog class. Provides a dialog for the settings
  * of a legend describing markers or lines.
  * @author Werner Stille
  */
class LegendDlg : public QDialog {
  Q_OBJECT

public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param model pointer to KplDoc model object.
   * @param ld pointer to legend data structure.
   */
  LegendDlg(QWidget *parent, KplDoc *model, PLEGENDDATA ld);

protected:
  /**
   * Copies settings to legend data structure.
   * @param accept true for leaving the dialog, false for applying.
   */
  void getValues(bool accept);

protected slots:
  /** Displays dialog to edit text. */
  void slotEditText();
  /** Displays dialog to select marker or line type. */
  void slotSymbol();
  /** Applies settings and leaves the dialog. */
  void slotOK();
  /** Applies settings without leaving the dialog. */
  void slotApply();
  /** Displays help for the legend settings dialog. */
  void slotHelp();

signals:
  /**
    * Emitted when applied changes require updating the menus and repainting the
    * page view without leaving the dialog.
    */
  void modelChanged();

private:
  KplDoc *m;
  PLEGENDDATA ld;
  QLineEdit *ex0, *ey0, *eText, *exl, *eyl;
  QSpinBox *symb, *eRelSize;
  KColorButton *colSymb, *colText;
};

#endif
