/***************************************************************************
                          utils.cpp  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sun Apr 25 1999
    copyright            : (C) 1999 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qlineedit.h>
#include <qmultilinedit.h>
#include <qfileinfo.h>
#include <qdir.h>
#include <kapp.h>
#include <kfm.h>
#include <kfiledialog.h>
#include <kmsgbox.h>
#include "kpldoc.h"
#include "funcdlg.h"

#ifndef min
  #define min(x,y) ((x) < (y) ? (x) : (y))
#endif

bool getVal(QWidget *parent, QLineEdit *edit, double *val,
            const char *errmsg)
{
  bool ok = true;
  QString s;

  s = edit->text();
  *val = s.toDouble(&ok);
  if (!ok)
    KMsgBox::message(parent, i18n("Error"), errmsg, 2);
  return ok;
}

void getFile(QLineEdit *e, KplDoc *m)
{
  QString s = KFileDialog::getOpenFileName(m->currentDir(), "*.so");
  if (!s.isEmpty()) {
    e->setText(s);
    QFileInfo fi(s);
    m->setCurrentDir(fi.dirPath(true));
  }
}

void getFunc(QWidget *parent, QLineEdit *name, QLineEdit *func)
{
  FILE *f;

  QFileInfo fi(name->text());
  QString path = fi.dirPath(true) + "/" + fi.baseName() + ".def";
  if ((f = fopen(path, "r"))) {
    ChooseFuncDlg dlg(parent, f, func);
    dlg.exec();
  } else
    KMsgBox::message(parent, i18n("Error"),
                     i18n("while trying to open module definition file"),
                     KMsgBox::EXCLAMATION);
}

void getPar(QWidget *parent, double *p, KplDoc *m)
{
  int i;
  char s[81];
  FILE *f;

  QString url = KFileDialog::getOpenFileURL(m->currentDir(), "*.par");
  if (!url.isEmpty()) {
    QString fn;
    if (KFM::download(url, fn)) {
      if ((f = fopen(fn, "r"))) {
        for (i = 0; i < NPMAX; i++)
          p[i] = 0.0;
        i = -1;
        while (fgets(s, 80, f) && (++i < NPMAX))
          if (sscanf(s, "%lg", &p[i]) != 1)
            break;
        fclose(f);
        KURL u = KURL(url);
        if (u.isLocalFile()) {
          QFileInfo fi(fn);
          m->setCurrentDir(fi.dirPath(true));
        } else
          m->setCurrentDir(u.directoryURL());
      } else
        KMsgBox::message(parent, i18n("Error"),
                         i18n("while trying to open parameter file"), 2);
      KFM::removeTempFile(fn);
    }
  }
}

void savePar(QWidget *parent, double *p, KplDoc *m)
{
  int i;
  FILE *f;

  QString url = KFileDialog::getSaveFileURL(m->currentDir(), "*.par");
  if (!url.isEmpty()) {
    KURL u = KURL(url);
    if (u.isLocalFile()) {
      url = u.path();
      QFileInfo fi(url);
      if (fi.exists())
        if (KMsgBox::yesNo(parent, i18n("Warning"),
                           i18n("A file with this name exists already.\n"
                                "Do you want to overwrite it?"),
                           KMsgBox::EXCLAMATION) != 1)
          return;
      f = fopen(url, "w");
    } else
      f = fopen(QDir::homeDirPath() + "/kpltmp.tmp", "w");
    if (f) {
      for (i = 0; i < NPMAX; i++)
        fprintf(f, "%g\n", p[i]);
      fclose(f);
      if (u.isLocalFile()) {
        QFileInfo fi(url);
        m->setCurrentDir(fi.dirPath(true));
      } else {
        m->setCurrentDir(u.directoryURL());
        m->kfm = new KFM;
        m->connectKFM();
        m->kfm->copy(QDir::homeDirPath() + "/kpltmp.tmp", url);
      }
    } else
      KMsgBox::message(parent, i18n("Error"),
                       i18n("while trying to open parameter file"), 2);
  }
}

void showSource(QMultiLineEdit *ml, QFile *f, const char *func)
{
  int i = 0, line = 0, col = 0;
  QString s;

  ml->setAutoUpdate(false);
  ml->setFont(kapp->fixedFont);
  QTextStream t(f);
  while (!t.eof()) {
    s = t.readLine();
    if (func && !line) {
      col = s.find(func);
      if (col != -1)
        line = i;
    }
    ml->append(s);
    i++;
  }
  f->close();
  ml->setAutoUpdate(true);
  ml->repaint();
  if (line) {
    ml->setCursorPosition(line, col);
    ml->setCursorPosition(line, col + strlen(func), true);
  }
  ml->setReadOnly(true);
  ml->setFocus();
}

